@@ -12,10 +12,6 @@
 // initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
 // initial version: 02/27/2002
 //
-// This file contains Patent Pending Technology P2Pnat Technology
-// under special royalty-free license to the GnuGk Project.
-//
-// P2Pnat implementation written by Simon Horne <s.horne@packetizer.com>
 //////////////////////////////////////////////////////////////////
 
 #include <ptlib.h>
@@ -30,12 +26,16 @@
 #include "cisco.h"
 #include "GkClient.h"
 
-#ifdef P2PnatClient
+#ifdef HAS_H46023
   #include <h460/h4601.h>
   #include <ptclib/pstun.h>
   #include <ptclib/random.h>
 #endif
 
+#if P_DNS
+  #include <ptclib/pdns.h>
+#endif
+
 using std::vector;
 using std::multimap;
 using std::make_pair;
@@ -53,6 +53,7 @@ class AlternateGKs {
 public:
 	AlternateGKs(const PIPSocket::Address &, WORD);
 	void Set(const H225_ArrayOf_AlternateGK &);
+	void Set(const PString &);
 	bool Get(PIPSocket::Address &, WORD &);
 
 private:
@@ -77,6 +78,18 @@ void AlternateGKs::Set(const H225_ArrayOf_AlternateGK & agk)
 	index = AltGKs.begin();
 }
 
+void AlternateGKs::Set(const PString & addr)
+{
+	PIPSocket::Address gkaddr;
+	WORD gkport;
+	if (GetTransportAddress(addr, GK_DEF_UNICAST_RAS_PORT, gkaddr, gkport)) {
+		H323TransportAddress taddr(gkaddr, gkport);
+		H225_TransportAddress haddr;
+		taddr.SetPDU(haddr);
+		AltGKs.insert(make_pair(AltGKs.size()+1, haddr));
+	}
+}
+
 bool AlternateGKs::Get(PIPSocket::Address & gkaddr, WORD & gkport)
 { 
 	if (!AltGKs.empty()) {
@@ -152,6 +165,7 @@ void NATClient::Exec()
 			return;
 		}
 	}
+	PWaitAndSignal lockDeletion(m_deletionPreventer);
 	delete socket;
 	socket = NULL;
 	// If we lose the TCP connection then retry after 60 sec
@@ -186,12 +200,14 @@ void NATClient::SendInfo(int state)
 	information.SetIE(Q931::FacilityIE, epid);
 	information.SetCallState(Q931::CallStates(state));
 	information.Encode(buf);
-	socket->TransmitData(buf);
+	if (socket) {
+		socket->TransmitData(buf);
+	}
 }
 
 //////////////////////////////////////////////////////////////////////
 
-#ifdef P2PnatClient
+#ifdef HAS_H46023
 
 // stuff cut from pstun.cxx
 
@@ -430,7 +446,7 @@ public:
   }
 
 
-  bool Read(PUDPSocket & socket)
+  bool Read(UDPSocket & socket)
   {
     if (!socket.Read(GetPointer(1000), 1000))
       return false;
@@ -438,12 +454,12 @@ public:
     return true;
   }
   
-  bool Write(PUDPSocket & socket) const
+  bool Write(UDPSocket & socket) const
   {
     return socket.Write(theArray, ((STUNmessageHeader *)theArray)->msgLength+sizeof(STUNmessageHeader)) != FALSE;
   }
 
-  bool Poll(PUDPSocket & socket, const STUNmessage & request, PINDEX pollRetries)
+  bool Poll(UDPSocket & socket, const STUNmessage & request, PINDEX pollRetries)
   {
     for (PINDEX retry = 0; retry < pollRetries; retry++) {
       if (!request.Write(socket))
@@ -536,6 +552,9 @@ class STUNClient   :  public  Job,
 			const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny()
     );
 
+protected:
+	bool OpenSocketA(UDPSocket & socket, PortInfo & portInfo, const PIPSocket::Address & binding);
+
 private:
 	// override from class Task
 	virtual void Exec();
@@ -624,7 +643,41 @@ void STUNClient::OnDetectedNAT(int nattype)
 
 	// Call back to signal the GKClient to do a lightweight reregister
 	// to notify the gatekeeper
-    m_client->P2Pnat_TypeDetected(nattype);
+    m_client->H46023_TypeDetected(nattype);
+}
+
+bool STUNClient::OpenSocketA(UDPSocket & socket, PortInfo & portInfo, const PIPSocket::Address & binding)
+{
+  if (serverPort == 0) {
+    PTRACE(1, "STUN\tServer port not set.");
+    return false;
+  }
+
+  if (!PIPSocket::GetHostAddress(serverHost, cachedServerAddress) || !cachedServerAddress.IsValid()) {
+    PTRACE(2, "STUN\tCould not find host \"" << serverHost << "\".");
+    return false;
+  }
+
+  PWaitAndSignal mutex(portInfo.mutex);
+
+  WORD startPort = portInfo.currentPort;
+
+  do {
+    portInfo.currentPort++;
+    if (portInfo.currentPort > portInfo.maxPort)
+      portInfo.currentPort = portInfo.basePort;
+
+    if (socket.Listen(binding, 1, portInfo.currentPort)) {
+      socket.SetSendAddress(cachedServerAddress, serverPort);
+      socket.SetReadTimeout(replyTimeout);
+      return true;
+    }
+
+  } while (portInfo.currentPort != startPort);
+
+  PTRACE(1, "STUN\tFailed to bind to local UDP port in range "
+         << portInfo.currentPort << '-' << portInfo.maxPort);
+  return false;
 }
 
 bool STUNClient::CreateSocketPair(UDPSocket * & rtp, UDPSocket * & rtcp, const PIPSocket::Address & binding)
@@ -650,8 +703,8 @@ bool STUNClient::CreateSocketPair(UDPSocket * & rtp, UDPSocket * & rtcp, const P
   for (i = 0; i < numSocketsForPairing; i++)
   {
     PINDEX idx = stunSocket.Append(new STUNsocket);
-    if (!OpenSocket(stunSocket[idx], pairedPortInfo, binding)) {
-      PTRACE(1, "STUN\tUnable to open socket to server " << serverAddress);
+    if (!OpenSocketA(stunSocket[idx], pairedPortInfo, binding)) {
+      PTRACE(1, "STUN\tUnable to open socket to server " << GetServer());
       return false;
 	}
 
@@ -665,7 +718,7 @@ bool STUNClient::CreateSocketPair(UDPSocket * & rtp, UDPSocket * & rtcp, const P
   {
     if (!response[i].Poll(stunSocket[i], request[i], pollRetries))
     {
-      PTRACE(1, "STUN\tServer " << serverAddress << ':' << serverPort << " unexpectedly went offline.");
+      PTRACE(1, "STUN\tServer unexpectedly went offline." << GetServer());
       return false;
     }
   }
@@ -675,7 +728,7 @@ bool STUNClient::CreateSocketPair(UDPSocket * & rtp, UDPSocket * & rtcp, const P
     STUNmappedAddress * mappedAddress = (STUNmappedAddress *)response[i].FindAttribute(STUNattribute::MAPPED_ADDRESS);
     if (mappedAddress == NULL)
     {
-      PTRACE(2, "STUN\tExpected mapped address attribute from server " << serverAddress << ':' << serverPort);
+      PTRACE(2, "STUN\tExpected mapped address attribute from server " << GetServer());
       return false;
     }
     if (GetNatType(FALSE) != SymmetricNat)
@@ -847,7 +900,7 @@ GkClient::GkClient()
 	m_rewriteInfo(NULL), m_natClient(NULL),
 	m_parentVendor(ParentVendor_GnuGk), m_endpointType(EndpointType_Gateway),
 	m_discoverParent(true)
-#ifdef P2PnatClient
+#ifdef HAS_H46023
 	, m_nattype(0), m_natnotify(false), gk_H460_23(false),  m_stunClient(NULL)
 #endif
 {
@@ -931,7 +984,23 @@ void GkClient::OnReload()
 	
 	PIPSocket::Address gkaddr = m_gkaddr;
 	WORD gkport = m_gkport;
-	const PCaselessString gk(cfg->GetString(EndpointSection, "Gatekeeper", "no"));
+	PCaselessString gk(cfg->GetString(EndpointSection, "Gatekeeper", "no"));
+
+	PStringList gkHost;
+#if P_DNS
+	PStringList str;
+    if (PDNS::LookupSRV(gk,"_h323rs._udp.",str)) {
+		PTRACE(5, "EP\t" << str.GetSize() << " h323rs SRV Records found" );
+		  for (PINDEX i = 0; i < str.GetSize(); i++) {
+			PCaselessString newhost = str[i].Right(str[i].GetLength()-5);
+			PTRACE(4, "EP\th323rs SRV record " << newhost );
+			if (i == 0) 
+				gk = newhost;
+			else 
+				m_gkList->Set(newhost);
+		  }
+	} 
+#endif	
 	
 	if (!IsRegistered())
 		m_ttl = GkConfig()->GetInteger(EndpointSection, "TimeToLive", DEFAULT_TTL);
@@ -993,21 +1062,20 @@ bool GkClient::OnSendingRRQ(H225_RegistrationRequest &rrq)
 		}
 	}
 
-#ifdef P2PnatClient
+#ifdef HAS_H46023
 		// H.460.23 Feature
-	    bool NoP2Pfeature = Toolkit::AsBool(GkConfig()->GetString(EndpointSection, "DisableH.460.23", "1"));
-		if (!NoP2Pfeature) {
+		if (Toolkit::Instance()->IsH46023Enabled()) {
 			bool contents = false;
 			rrq.IncludeOptionalField(H225_RegistrationRequest::e_featureSet);
 			H460_FeatureStd feat = H460_FeatureStd(23); 
 
 			if (!IsRegistered()) {
-					feat.Add(P2P_RemoteNAT,H460_FeatureContent(true));
+					feat.Add(Std23_RemoteNAT,H460_FeatureContent(true));
 					contents = true;
 			} else {
 					int natType = 0;
-					if (P2Pnat_TypeNotify(natType)) {
-						feat.Add(P2P_NATdet,H460_FeatureContent(natType,8)); 
+					if (H46023_TypeNotify(natType)) {
+						feat.Add(Std23_NATdet,H460_FeatureContent(natType,8)); 
 						contents = true;
 					}
 			}
@@ -1177,7 +1245,7 @@ bool GkClient::SendARQ(Routing::AdmissionRequest & arq_obj)
 		arq.m_callIdentifier = oarq.m_callIdentifier;
 	}
 
-#ifdef P2PnatClient
+#ifdef HAS_H46023
 	if (gk_H460_23) {
 		arq.IncludeOptionalField(H225_AdmissionRequest::e_featureSet);
 			H460_FeatureStd feat = H460_FeatureStd(24); 
@@ -1275,13 +1343,13 @@ bool GkClient::SendARQ(Routing::SetupRequest & setup_obj, bool answer, int natof
 	// workaround for bandwidth, as OpenH323 library :p
 	arq.m_bandWidth = 1280;
 
-#ifdef P2PnatClient
+#ifdef HAS_H46023
 	if (gk_H460_23) {
 		arq.IncludeOptionalField(H225_AdmissionRequest::e_featureSet);
 		H460_FeatureStd feat = H460_FeatureStd(24); 
 
 		if (answer  && natoffload > 0) 
-			feat.Add(P2P_NATInstruct,H460_FeatureContent((unsigned)natoffload,8));
+			feat.Add(Std24_NATInstruct,H460_FeatureContent((unsigned)natoffload,8));
 
 		arq.m_featureSet.IncludeOptionalField(H225_FeatureSet::e_supportedFeatures);
 		H225_ArrayOf_FeatureDescriptor & desc = arq.m_featureSet.m_supportedFeatures;
@@ -1626,15 +1694,16 @@ bool GkClient::WaitForACF(H225_AdmissionRequest &arq, RasRequester & request, Ro
 				Route route("parent", acf.m_destCallSignalAddress);
 				route.m_flags |= Route::e_toParent;
 				robj->AddRoute(route);
-#ifdef P2PnatClient
+#ifdef HAS_H46023
+			  if (Toolkit::Instance()->IsH46023Enabled()) {
 				if (acf.HasOptionalField(H225_AdmissionConfirm::e_featureSet)) {
-				  callptr call = arq.HasOptionalField(H225_AdmissionRequest::e_callIdentifier) ?
-					  CallTable::Instance()->FindCallRec(arq.m_callIdentifier) : CallTable::Instance()->FindCallRec(arq.m_callReferenceValue);
-						H460_FeatureSet fs = H460_FeatureSet(acf.m_featureSet);
-						int rNaTFS = 23; 
-						if (fs.HasFeature(rNaTFS)) 
-								HandleP2P_ACF(call, (H460_FeatureStd *)fs.GetFeature(rNaTFS));
-			     }
+					callptr call = arq.HasOptionalField(H225_AdmissionRequest::e_callIdentifier) ?
+						CallTable::Instance()->FindCallRec(arq.m_callIdentifier) : CallTable::Instance()->FindCallRec(arq.m_callReferenceValue);
+					H460_FeatureSet fs = H460_FeatureSet(acf.m_featureSet);
+					if (fs.HasFeature(24)) 
+						HandleP2P_ACF(call, (H460_FeatureStd *)fs.GetFeature(24));
+				 }
+			  }
 #endif
 			}
 			return true;
@@ -1706,32 +1775,33 @@ void GkClient::OnRCF(RasMsg *ras)
 		}
 	}
 
-#ifdef P2PnatClient
-	if (rcf.HasOptionalField(H225_RegistrationConfirm::e_genericData)) {
-		  H460_FeatureSet fs = H460_FeatureSet(rcf.m_genericData);
-		   int rNaTFS = 23; 
-		   if (fs.HasFeature(rNaTFS)) 
-			   HandleP2P_RCF((H460_FeatureStd *)fs.GetFeature(rNaTFS));
+#ifdef HAS_H46023
+    if (Toolkit::Instance()->IsH46023Enabled()) {
+		if (rcf.HasOptionalField(H225_RegistrationConfirm::e_genericData)) {
+			H460_FeatureSet fs = H460_FeatureSet(rcf.m_genericData);
+			if (fs.HasFeature(23)) 
+				HandleP2P_RCF((H460_FeatureStd *)fs.GetFeature(23));
+		}
 	}
 #endif
 }
 
-#ifdef P2PnatClient
+#ifdef HAS_H46023
 void GkClient::HandleP2P_RCF(H460_FeatureStd * feat)
 {
-   PBoolean proxy = FALSE;
+//   PBoolean proxy = FALSE;
    PBoolean NATdetect = FALSE;
 
    gk_H460_23 = true;
 
-   if (feat->Contains(P2P_ProxyNAT)) 
-       proxy = feat->Value(P2P_ProxyNAT);
+//   if (feat->Contains(Std24_ProxyNAT)) 
+//       proxy = feat->Value(Std24_ProxyNAT);
 
-   if (feat->Contains(P2P_IsNAT))
-       NATdetect = feat->Value(P2P_IsNAT);
+   if (feat->Contains(Std23_IsNAT))
+       NATdetect = feat->Value(Std23_IsNAT);
 
-   if (feat->Contains(P2P_DetRASAddr)) {
-       H323TransportAddress addr = feat->Value(P2P_DetRASAddr);
+   if (feat->Contains(Std23_DetRASAddr)) {
+       H323TransportAddress addr = feat->Value(Std23_DetRASAddr);
 	   if (!NATdetect) {
 		   PIPSocket::Address ip;
 		   addr.GetIpAddress(ip);
@@ -1739,13 +1809,13 @@ void GkClient::HandleP2P_RCF(H460_FeatureStd * feat)
 	   }
    }
 
-   if (NATdetect && feat->Contains(P2P_STUNAddr)) {
-           H323TransportAddress addr = feat->Value(P2P_STUNAddr);
+   if (NATdetect && feat->Contains(Std23_STUNAddr)) {
+           H323TransportAddress addr = feat->Value(Std23_STUNAddr);
 		   m_stunClient = new STUNClient(this,addr);
    }
 }
 
-bool GkClient::P2Pnat_TypeNotify(int & nattype)
+bool GkClient::H46023_TypeNotify(int & nattype)
 {
 	if (m_natnotify) 
 		nattype = m_nattype;
@@ -1755,10 +1825,10 @@ bool GkClient::P2Pnat_TypeNotify(int & nattype)
 
 void GkClient::HandleP2P_ACF(callptr m_call, H460_FeatureStd * feat)
 {
-    if (feat->Contains(P2P_NATInstruct)) {
-		unsigned NATinst = feat->Value(P2P_NATInstruct); 
+    if (feat->Contains(Std24_NATInstruct)) {
+		unsigned NATinst = feat->Value(Std24_NATInstruct); 
 
-        PTRACE(4,"GKC\tP2Pnat strategy for call set to " << NATinst);
+        PTRACE(4,"GKC\tH46024 strategy for call set to " << NATinst);
 
 		if (m_call) 
 			m_call->SetNATStrategy((CallRec::NatStrategy)NATinst);
@@ -1766,7 +1836,7 @@ void GkClient::HandleP2P_ACF(callptr m_call, H460_FeatureStd * feat)
 	}
 }
 
-bool GkClient::P2Pnat_CreateSocketPair(const H225_CallIdentifier & id, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated)
+bool GkClient::H46023_CreateSocketPair(const H225_CallIdentifier & id, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated)
 {
 
   if (!m_stunClient) 
@@ -1781,7 +1851,7 @@ bool GkClient::P2Pnat_CreateSocketPair(const H225_CallIdentifier & id, UDPProxyS
 	return false;
 }
 
-void GkClient::P2Pnat_TypeDetected(int nattype)
+void GkClient::H46023_TypeDetected(int nattype)
 {
 	m_nattype = nattype;
 	m_natnotify = true;
