@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKCLIENT_H
-#define GKCLIENT_H "@(#) $Id: GkClient.h,v 1.35 2009/05/28 12:06:07 willamowius Exp $"
+#define GKCLIENT_H "@(#) $Id: GkClient.h,v 1.39 2010/04/29 22:22:12 willamowius Exp $"
 
 #include "gk_const.h"
 #include "Toolkit.h"
@@ -36,7 +36,7 @@ class H225_Setup_UUIE;
 class H225_ArrayOf_ClearToken;
 class H225_ArrayOf_CryptoH323Token;
 
-#ifdef hasH460
+#ifdef HAS_H460
   class H460_FeatureStd;
   class STUNClient;
   class UDPProxySocket;
@@ -65,7 +65,7 @@ public:
 	void OnReload();
 	void CheckRegistration();
 	bool CheckFrom(const RasMsg *) const;
-	bool CheckFrom(const PIPSocket::Address & ip) { return m_gkaddr == ip; }
+	bool CheckFrom(const PIPSocket::Address & ip) const { return m_gkaddr == ip; }
 	bool IsRegistered() const { return m_registered; }
 	bool IsNATed() const { return m_natClient != NULL; }
 	PString GetParent() const;
@@ -120,23 +120,24 @@ public:
 	{
 
 #ifdef OpenH323Factory
-         for (PINDEX i = 0; i < m_h235Authenticators->GetSize();  i++) {
-             H235Authenticator * authenticator = (H235Authenticator *)(*m_h235Authenticators)[i].Clone();
+		for (PINDEX i = 0; i < m_h235Authenticators->GetSize();  i++) {
+			H235Authenticator * authenticator = (H235Authenticator *)(*m_h235Authenticators)[i].Clone();
 
-		   if (authenticator->IsSecuredPDU(rasmsg.GetTag(), FALSE)) {
-		     authenticator->SetLocalId(id);
-		     authenticator->SetPassword(m_password);
+			if (authenticator->IsSecuredPDU(rasmsg.GetTag(), FALSE)) {
+				authenticator->SetLocalId(id);
+				authenticator->SetPassword(m_password);
 
-             if (authenticator->PrepareTokens(rasmsg.m_tokens, rasmsg.m_cryptoTokens)) {
-                 PTRACE(4, "GKClient\tPrepared PDU with authenticator " << authenticator);
-	 		 }
-		   }
+				if (authenticator->PrepareTokens(rasmsg.m_tokens, rasmsg.m_cryptoTokens)) {
+					PTRACE(4, "GKClient\tPrepared PDU with authenticator " << authenticator);
+				}
+			}
+			delete authenticator;
 		}
-          if (rasmsg.m_tokens.GetSize() > 0)
-              rasmsg.IncludeOptionalField(RAS::e_tokens);
+		if (rasmsg.m_tokens.GetSize() > 0)
+			rasmsg.IncludeOptionalField(RAS::e_tokens);
 
-          if (rasmsg.m_cryptoTokens.GetSize() > 0)
-              rasmsg.IncludeOptionalField(RAS::e_cryptoTokens);
+		if (rasmsg.m_cryptoTokens.GetSize() > 0)
+			rasmsg.IncludeOptionalField(RAS::e_cryptoTokens);
 #else
 		if (!m_password) {
 			// to avoid including h235.h
@@ -246,13 +247,13 @@ private:
 	H235Authenticators* m_h235Authenticators;
 #endif
 
-#ifdef P2PnatClient
-	// Handle P2Pnat RCF
+#ifdef HAS_H46023
+	// Handle H46023 RCF
 	void HandleP2P_RCF(H460_FeatureStd * feat);
-    // Handle P2Pnat ACF
+    // Handle H46023 ACF
 	void HandleP2P_ACF(callptr m_call, H460_FeatureStd * feat);
     // Notify NAT type
-    bool P2Pnat_TypeNotify(int & nattype);
+    bool H46023_TypeNotify(int & nattype);
 	// detected NAT type
 	int m_nattype;
 	// notify of NAT type
@@ -264,9 +265,9 @@ private:
 
 public:
 	// NAT type detected
-	void P2Pnat_TypeDetected(int nattype);
+	void H46023_TypeDetected(int nattype);
     // Create socket pair
-    bool P2Pnat_CreateSocketPair(const H225_CallIdentifier & id, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated);
+    bool H46023_CreateSocketPair(const H225_CallIdentifier & id, UDPProxySocket * & rtp, UDPProxySocket * & rtcp, bool & nated);
 
 #endif
 };
