@@ -11,7 +11,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKSTATUS_H
-#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.32 2009/05/15 07:33:14 willamowius Exp $"
+#define GKSTATUS_H "@(#) $Id: GkStatus.h,v 1.34 2010/04/06 07:05:33 willamowius Exp $"
 
 #include <map>
 #include "yasocket.h"
@@ -124,6 +124,7 @@ public:
 		e_Shutdown,                    /// Shutdown the program
 		e_RouteToAlias,                /// Route a call to a specified alias
 		e_RouteToGateway,              /// Route a call to a specified alias + destinationCallSignalAddr
+		e_BindAndRouteToGateway,       /// Route a call to a specified alias + destinationCallSignalAddr + specify bind IP
 		e_RouteReject,                 /// Reject to Route a call upon ARQ (send ARJ)
 		e_Trace,                       /// change trace level for status interface output
 #if PTRACING
@@ -142,6 +143,7 @@ public:
 		e_SendProceeding,              /// Send a CallProceeding message for this call-id
 		e_GetAuthInfo,                 /// Get auth module information
 		e_GetAcctInfo,                 /// Get acct module information
+		e_ResetCallCounters,           /// Reset the call counters
 		e_numCommands
 		/// Number of different strings
 	};
