@@ -31,7 +31,7 @@
 #include "Neighbor.h"
 #include "config.h"
 
-#ifdef hasH460
+#ifdef HAS_H460
 	#include <h460/h4601.h>
 #endif
 
@@ -51,56 +51,29 @@ const char *LRQFeaturesSection = "RasSrv::LRQFeatures";
 
 static const char OID_MD5[] = "1.2.840.113549.2.5";
 
-static PWCharArray GetUCS2plusNULL(const PString & str)
-{
-	PWCharArray ucs2 = str.AsUCS2();
-	PINDEX len = ucs2.GetSize();
-	if (len > 0 && ucs2[len-1] != 0)
-	  ucs2.SetSize(len+1);
-	return ucs2;
-}
-
-// helper function to calculate GKPwdHash (could migrate into H323Plus)
 void SetCryptoGkTokens(H225_ArrayOf_CryptoH323Token & cryptoTokens, const PString & id, const PString & password)
 {
-	// TODO: at least Tandberg seems to calculate the GKPwdHash differently
-	cryptoTokens.RemoveAll();
-	// Cisco compatible hash calculation
-	H235_ClearToken clearToken;
+	cryptoTokens.SetSize(0);
 
-	// fill the PwdCertToken to calculate the hash
-	clearToken.m_tokenOID = "0.0";
+	H235AuthSimpleMD5 auth;
+	// avoid copying for thread-safely
+	auth.SetLocalId((const char *)id);
+	auth.SetPassword((const char *)password);
+	H225_ArrayOf_ClearToken dumbTokens;
+	H225_ArrayOf_CryptoH323Token newCryptoTokens;
+	auth.PrepareTokens(dumbTokens, newCryptoTokens);
+	H225_CryptoH323Token_cryptoEPPwdHash & cryptoEPPwdHash = newCryptoTokens[0];
 
-	clearToken.IncludeOptionalField(H235_ClearToken::e_generalID);
-	clearToken.m_generalID = GetUCS2plusNULL(id);
-
-	clearToken.IncludeOptionalField(H235_ClearToken::e_password);
-	clearToken.m_password = GetUCS2plusNULL(password);
-
-	clearToken.IncludeOptionalField(H235_ClearToken::e_timeStamp);
-	clearToken.m_timeStamp = (int)time(NULL);
-
-	// Encode it into PER
-	PPER_Stream strm;
-	clearToken.Encode(strm);
-	strm.CompleteEncoding();
-
-	// Generate an MD5 of the clear tokens PER encoding.
-	PMessageDigest5 stomach;
-	stomach.Process(strm.GetPointer(), strm.GetSize());
-	PMessageDigest5::Code digest;
-	stomach.Complete(digest);
-
-	// Create the H.225 crypto token
+	// Create the H.225 GK crypto token
 	H225_CryptoH323Token * finalCryptoToken = new H225_CryptoH323Token;
 	finalCryptoToken->SetTag(H225_CryptoH323Token::e_cryptoGKPwdHash);
 	H225_CryptoH323Token_cryptoGKPwdHash & cryptoGKPwdHash = *finalCryptoToken;
 
 	// Set the token data that actually goes over the wire
 	cryptoGKPwdHash.m_gatekeeperId = Toolkit::GKName();
-	cryptoGKPwdHash.m_timeStamp = clearToken.m_timeStamp;
+	cryptoGKPwdHash.m_timeStamp = cryptoEPPwdHash.m_timeStamp;
 	cryptoGKPwdHash.m_token.m_algorithmOID = OID_MD5;
-	cryptoGKPwdHash.m_token.m_hash.SetData(sizeof(digest)*8, (const BYTE *)&digest);
+	cryptoGKPwdHash.m_token.m_hash = cryptoEPPwdHash.m_token.m_hash;
 	
 	if (finalCryptoToken != NULL)
 		cryptoTokens.Append(finalCryptoToken);
@@ -168,7 +141,7 @@ template<class R>
 class LRQSender : public LRQFunctor {
 public:
 	LRQSender(const R & r) : m_r(r) {}
-	virtual PrefixInfo operator()(Neighbor *, WORD) const;
+	virtual PrefixInfo operator()(Neighbor *, WORD seqnum) const;
 
 private:
 	const R & m_r;
@@ -306,6 +279,39 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 		PStringArray p(sprefixes[i].Tokenise(":=", false));
 		m_sendPrefixes[p[0]] = (p.GetSize() > 1) ? p[1].AsInteger() : 1;
 	}
+
+	PString salias(config->GetString(section, "SendAliases", ""));
+	PStringArray defs(salias.Tokenise(",", FALSE));
+	m_sendAliases.SetSize(0);
+	for (PINDEX i = 0; i < defs.GetSize(); i++) {
+		if (defs[i].Find("-") != P_MAX_INDEX) {
+			// range
+			PStringArray bounds(defs[i].Tokenise("-", FALSE));
+			unsigned lower = bounds[0].AsUnsigned();
+			unsigned upper = 0;
+			if (bounds.GetSize() == 2) {
+				upper = bounds[1].AsUnsigned();
+			} else {
+				PTRACE(1, "SendAliases: Invalid range definition: " << defs[i]);
+				continue;
+			}
+			if (upper <= lower) {
+				PTRACE(1, "SendAliases: Invalid range bounds: " << defs[i]);
+				continue;
+			}
+			unsigned num = upper - lower;
+			for (unsigned j = 0; j <= num; j++) {
+				PString number(lower + j);
+				PTRACE(4, "Adding alias " << number << " to neighbor " << m_id << " (from range)");
+				m_sendAliases.AppendString(number);
+			}
+		} else {
+			// single alias
+			PTRACE(4, "Adding alias " << defs[i] << " to neighbor " << m_id);
+			m_sendAliases.AppendString(defs[i]);
+		}
+	}
+
 	PString aprefix(config->GetString(section, "AcceptPrefixes", "*"));
 	m_acceptPrefixes = PStringArray(aprefix.Tokenise(",", false));
 	if (m_keepAliveTimer != GkTimerManager::INVALID_HANDLE)
@@ -362,13 +368,15 @@ void Neighbor::SetH46018GkKeepAliveInterval(int interval)
 		m_keepAliveTimerInterval = interval;
 		if (m_keepAliveTimer != GkTimerManager::INVALID_HANDLE)
 			Toolkit::Instance()->GetTimerManager()->UnregisterTimer(m_keepAliveTimer);
-		PTime now;
-		if (m_keepAliveTimerInterval > 0)
+		if (m_keepAliveTimerInterval > 0) {
+			PTime now;
 			m_keepAliveTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
 				this, &Neighbor::SendH46018GkKeepAlive, now, m_keepAliveTimerInterval);	// do it now and every n seconds
+		}
 	}
 }
 
+// initialize neighbor object created by SRV policy
 bool Neighbor::SetProfile(const PString & name, const H323TransportAddress & addr)
 {
   addr.GetIpAndPort(m_ip,m_port);
@@ -378,12 +386,7 @@ bool Neighbor::SetProfile(const PString & name, const H323TransportAddress & add
   m_externalGK = true;
 
   m_sendPrefixes.clear();
-  PString sprefix("*");
-  PStringArray sprefixes = sprefix.Tokenise(",", false);
-	for (PINDEX i = 0; i < sprefixes.GetSize(); ++i) {
-		PStringArray p(sprefixes[i].Tokenise(":=", false));
-		m_sendPrefixes[p[0]] = (p.GetSize() > 1) ? p[1].AsInteger() : 1;
-	}
+  m_sendPrefixes["*"] = 1;
  
   SetForwardedInfo(LRQFeaturesSection);
 
@@ -396,6 +399,7 @@ PrefixInfo Neighbor::GetPrefixInfo(const H225_ArrayOf_AliasAddress & aliases, H2
 	Prefixes::iterator iter, biter = m_sendPrefixes.begin(), eiter = m_sendPrefixes.end();
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
 		H225_AliasAddress & alias = aliases[i];
+		// send by alias type
 		iter = m_sendPrefixes.find(alias.GetTagName());
 		if (iter != eiter) {
 			dest.SetSize(1);
@@ -403,6 +407,15 @@ PrefixInfo Neighbor::GetPrefixInfo(const H225_ArrayOf_AliasAddress & aliases, H2
 			return PrefixInfo(100, (short)iter->second);
 		}
 		PString destination(AsString(alias, false));
+		// send by exact alias match
+		for (PINDEX j = 0; j < m_sendAliases.GetSize(); j++) {
+			if (destination == m_sendAliases[j]) {
+				dest.SetSize(1);
+				dest[0] = alias;
+				return PrefixInfo(100, 1);
+			}
+		}
+		// send by prefix
 		while (iter != biter) {
 			--iter; // search in reverse order
 			const int len = MatchPrefix(destination, iter->first);
@@ -416,6 +429,7 @@ PrefixInfo Neighbor::GetPrefixInfo(const H225_ArrayOf_AliasAddress & aliases, H2
 			}
 		}
 	}
+	// send allways ? (handled last, treated as shortest match)
 	iter = m_sendPrefixes.find("*");
 	if (iter == eiter)
 		return nomatch;
@@ -587,24 +601,27 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const AdmissionRequest & re
 		lrq.m_canMapAlias = arq.m_canMapAlias;
 	}
 
-#ifdef hasH460
-       lrq.IncludeOptionalField(H225_LocationRequest::e_genericData); 
-       H225_ArrayOf_GenericData & data = lrq.m_genericData;
-       PINDEX lastPos = 0;
-       
-	   /// STD24  'NAT Support
-	   H460_FeatureStd std24 = H460_FeatureStd(24);
-	   lastPos++;
-	   data.SetSize(lastPos);
-	   data[lastPos-1] = std24;
+#ifdef HAS_H460
+	lrq.IncludeOptionalField(H225_LocationRequest::e_genericData); 
+	H225_ArrayOf_GenericData & data = lrq.m_genericData;
+	PINDEX lastPos = 0;
 
-	   /// OID9  'Remote application info
-	   H460_FeatureOID foid9 = H460_FeatureOID(OID9);
-	   lastPos++;
-	   data.SetSize(lastPos);
-	   data[lastPos-1] = foid9;
-	     		
+#ifdef HAS_H46023
+	if (Toolkit::Instance()->IsH46023Enabled()) {
+		H460_FeatureStd std24 = H460_FeatureStd(24);
+		lastPos++;
+		data.SetSize(lastPos);
+		data[lastPos-1] = std24;
+	}
 #endif
+
+	/// OID9  'Remote application info
+	H460_FeatureOID foid9 = H460_FeatureOID(OID9);
+	lastPos++;
+	data.SetSize(lastPos);
+	data[lastPos-1] = foid9;	
+#endif
+
 	return true;
 }
 
@@ -1068,7 +1085,16 @@ void NeighborList::OnReload()
 	}
 	for (i = 0; i < sz; ++i) {
 		const PString & nbid = cfgs.GetKeyAt(i);
-		const PString & type = cfgs.GetDataAt(i);
+		PString type = cfgs.GetDataAt(i);
+		// make neighbor type caseless
+		if (PCaselessString(type) == "GnuGK")
+			type = "GnuGK";
+		if (PCaselessString(type) == "CiscoGK")
+			type = "CiscoGK";
+		if (PCaselessString(type) == "ClarentGK")
+			type = "ClarentGK";
+		if (PCaselessString(type) == "GlonetGK")
+			type = "GlonetGK";
 		iter = find_if(m_neighbors.begin(), m_neighbors.end(),
 				compose1(bind2nd(equal_to<PString>(), nbid), mem_fun(&Neighbor::GetId))
 			      );
@@ -1248,13 +1274,13 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 	if (request.Send(m_neighbors)) {
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
-#ifdef hasH460
+#ifdef HAS_H460
 			if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
 			  H225_RasMessage ras;
 			  ras.SetTag(H225_RasMessage::e_locationConfirm);
               H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
 			  con = *lcf;
-			  route.m_destEndpoint = endptr(new EndpointRec(ras));	
+			  route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);	
 			}
 #endif
 			route.m_routeId = request.GetNeighborUsed();
@@ -1406,123 +1432,289 @@ public:
 	SRVPolicy() { m_name = "SRV"; }
 
 protected:
-    virtual bool OnRequest(SetupRequest &) { return false; }
     virtual bool OnRequest(FacilityRequest &) { return false; }
 
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
 
-private:
-	int m_Timeout;
+	virtual Route * CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly);
+	virtual Route * LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases);
+	virtual Route * LSLocalLookup(H225_ArrayOf_AliasAddress & aliases);
 };
 
-bool SRVPolicy::FindByAliases(
-	RoutingRequest &request,
-	H225_ArrayOf_AliasAddress &aliases
-	)
+Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
+		// only apply to urlID and h323ID
+		if ((aliases[i].GetTag() != H225_AliasAddress::e_url_ID)
+			&& (aliases[i].GetTag() != H225_AliasAddress::e_h323_ID))
+			continue;
 		PString alias(AsString(aliases[i], FALSE));
-	    if (alias.GetLength() == 0) continue;
-
-	// DNS SRV Record lookup
-		PString number;
-		PString domain;
 		PINDEX at = alias.Find('@');
-		if (at == P_MAX_INDEX) {
-		   number = "h323:t@" + alias;	
-		   domain = alias;
-	    } else {
-		   number = "h323:" + alias;
-		   domain = alias.Mid(at+1);
-		}
-	
+		// skip empty aliases or those without at-sign
+	    if ((alias.GetLength() == 0) || (at == P_MAX_INDEX))
+			continue;
+
+		// DNS SRV Record lookup
+		PString number = "h323:" + alias;
+		PString domain = alias.Mid(at+1);
+		PString localalias = alias.Left(at);
+		if (IsIPAddress(domain)
+			|| (domain.FindRegEx(PRegularExpression(":[0-9]+$", PRegularExpression::Extended)) != P_MAX_INDEX))
+			continue;	// don't use SRV record if domain part is IP or has port (Annex O, O.9), let dns policy handle them
+
 		// LS Record lookup
 		PStringList ls;
-			if (PDNS::LookupSRV(number,"_h323ls._udp.",ls)) {
-			 for (PINDEX i=0; i<ls.GetSize(); i++) {
+		if (PDNS::LookupSRV(number, "_h323ls._udp.", ls)) {
+			for (PINDEX i=0; i < ls.GetSize(); i++) {
 				PINDEX at = ls[i].Find('@');
 				PString ipaddr = ls[i].Mid(at + 1);
+				if (ipaddr.Left(7) == "0.0.0.0") {
+					PTRACE(1, "ROUTING\tERROR in LS SRV lookup (" << ls[i] << ")");
+					continue;
+				}
 				PTRACE(4, "ROUTING\tSRV LS located domain " << domain << " at " << ipaddr);
 				H323TransportAddress addr = H323TransportAddress(ipaddr);
 
-				// Create a SRV gatekeeper object
-				GnuGK * nb = new GnuGK();
-				if (!nb->SetProfile(domain,addr)) {
-					PTRACE(4, "ROUTING\tERROR setting SRV neighbor profile " << domain << " at " << addr);
-					return false;
+				PIPSocket::Address socketip;
+				WORD port;
+				if (!GetTransportAddress(ipaddr, GK_DEF_UNICAST_RAS_PORT, socketip, port) && socketip.IsValid()) {
+					PTRACE(1, "ROUTING\tERROR in SRV LS IP " << ipaddr);
+					continue;
 				}
+				if (Toolkit::Instance()->IsGKHome(socketip)) {
+					// this is my domain, no need to send LRQs, just look into the endpoint table
+					PINDEX numberat = number.Find('@');	// always has an @
+					H225_ArrayOf_AliasAddress find_aliases;
+					find_aliases.SetSize(1);
+					PString local_alias = number.Mid(5,numberat-5);
+					H323SetAliasAddress(local_alias, find_aliases[0]);
+					endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
+					if (ep) {
+						// endpoint found locally
+						Route * route = new Route(ep);
+						route->m_policy = m_name;
+						return route;
+					} else {
+						return NULL;
+					}
+				} else {
+					// Create a SRV gatekeeper object
+					GnuGK * nb = new GnuGK();
+					if (!nb->SetProfile(domain,addr)) {
+						PTRACE(4, "ROUTING\tERROR setting SRV neighbor profile " << domain << " at " << addr);
+						delete nb;
+						return NULL;
+					}
 
-				int m_neighborTimeout = GkConfig()->GetInteger(LRQFeaturesSection, "NeighborTimeout", 5) * 100;
+					int m_neighborTimeout = GkConfig()->GetInteger(LRQFeaturesSection, "NeighborTimeout", 5) * 100;
 
-				// Send LRQ to retreive callers signalling address 
-				LRQSender<AdmissionRequest> functor((AdmissionRequest &)request);
-				LRQRequester Request(functor);
-				if (Request.Send(nb)) {
-					if (H225_LocationConfirm *lcf = Request.WaitForDestination(m_neighborTimeout)) {
-							Route route(m_name, lcf->m_callSignalAddress);
-#ifdef hasH460
-			                if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
-			                    H225_RasMessage ras;
-			                    ras.SetTag(H225_RasMessage::e_locationConfirm);
-                                H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
-			                    con = *lcf;
-			                    route.m_destEndpoint = endptr(new EndpointRec(ras));	
-			                }
+					// Send LRQ to retreive callers signaling address
+					LRQSender<AdmissionRequest> functor((AdmissionRequest &)request);
+					LRQRequester Request(functor);
+					if (Request.Send(nb)) {
+						if (H225_LocationConfirm *lcf = Request.WaitForDestination(m_neighborTimeout)) {
+							Route * route = new Route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_H460
+							if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
+								H225_RasMessage ras;
+								ras.SetTag(H225_RasMessage::e_locationConfirm);
+								H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
+								con = *lcf;
+								route->m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+							}
 #endif
-							request.AddRoute(route);
-                             // Fix for calling only a numeric number so the @xxx is removed
-				             PString called = ls[i].Left(at);
-						     PINDEX k = 0;
-						     for (k = 0; k < called.GetLength(); ++k)
-								if (!isdigit(static_cast<unsigned char>(alias[k])))
-									break;
-								if (k >= alias.GetLength())
-									H323SetAliasAddress(called, aliases[i]);
-							 request.SetFlag(RoutingRequest::e_aliasesChanged);
-							return true;
-				    }
+							delete nb;
+							return route;
+						}
+					}
+					delete nb;
+					PTRACE(4, "ROUTING\tDNS SRV LRQ Error for " << domain << " at " << ipaddr);
+					// we found the directory for this domain, but it didn't have a destination, so we fail the call
+					Route * route = new Route();
+					route->m_flags |= Route::e_Reject;
+					return route;
 				}
-				PTRACE(4, "ROUTING\tDNS SRV LRQ Error for " << domain << " at " << ipaddr);
-			 }
-			}  
-
-        // CS SRV Lookup
-		PStringList cs;
-		   if (PDNS::LookupSRV(number,"_h323cs._tcp.",cs)) {
-			   for (PINDEX j=0; j<cs.GetSize(); j++) {
-				 PTRACE(4, "ROUTING\tSRV CS converted remote party " << alias << " to " << cs[j]);
-		         H225_TransportAddress dest;
-				 PINDEX in = cs[j].Find('@');
-                 PString dom = cs[j].Mid(in+1);
-		         if (GetTransportAddress(dom, GK_DEF_ENDPOINT_SIGNAL_PORT, dest)) {
-			       PIPSocket::Address addr;
-			       if (!(GetIPFromTransportAddr(dest, addr) && addr.IsValid()))
-				                    continue;
-			       Route route(m_name, dest);
-                    // Fix for calling only a numeric number so the @xxx is removed
-				     PString called = cs[j].Left(in);
-					  PINDEX l = 0;
-						for (l = 0; l < called.GetLength(); ++l)
-							if (!isdigit(static_cast<unsigned char>(alias[l])))
-								break;
-						if (l >= alias.GetLength())
-						  H323SetAliasAddress(called, aliases[i]);
-			       route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(dest);
-			       request.AddRoute(route);
-			       request.SetFlag(RoutingRequest::e_aliasesChanged);	
-				 }
-			   }
-		     return true;
-		   } 
-	} 
-  return false; 
+			}
+		}
+	}
+	return NULL;
 }
 
+Route * SRVPolicy::LSLocalLookup(H225_ArrayOf_AliasAddress & aliases)
+{
+	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
+		// only apply to urlID and h323ID
+		if ((aliases[i].GetTag() != H225_AliasAddress::e_url_ID)
+			&& (aliases[i].GetTag() != H225_AliasAddress::e_h323_ID))
+			continue;
+		PString alias(AsString(aliases[i], FALSE));
+		PINDEX at = alias.Find('@');
+		// skip empty aliases or those without at-sign
+	    if ((alias.GetLength() == 0) || (at == P_MAX_INDEX))
+			continue;
 
+		// DNS SRV Record lookup
+		PString number = "h323:" + alias;
+		PString domain = alias.Mid(at+1);
+		if (IsIPAddress(domain)
+			|| (domain.FindRegEx(PRegularExpression(":[0-9]+$", PRegularExpression::Extended)) != P_MAX_INDEX))
+			continue;	// don't use SRV record if domain part is IP or has port (Annex O, O.9), let dns policy handle them
+
+		// LS Record lookup
+		PStringList ls;
+		if (PDNS::LookupSRV(number, "_h323ls._udp.", ls)) {
+			for (PINDEX i=0; i < ls.GetSize(); i++) {
+				PINDEX at = ls[i].Find('@');
+				PString ipaddr = ls[i].Mid(at + 1);
+				if (ipaddr.Left(7) == "0.0.0.0") {
+					PTRACE(1, "ROUTING\tERROR in LS SRV lookup (" << ls[i] << ")");
+					continue;
+				}
+				PTRACE(4, "ROUTING\tSRV LS located domain " << domain << " at " << ipaddr);
+				H323TransportAddress addr = H323TransportAddress(ipaddr);
+
+				PIPSocket::Address socketip;
+				WORD port;
+				if (!GetTransportAddress(ipaddr, GK_DEF_UNICAST_RAS_PORT, socketip, port) && socketip.IsValid()) {
+					PTRACE(1, "ROUTING\tERROR in SRV LS IP " << ipaddr);
+					continue;
+				}
+				if (Toolkit::Instance()->IsGKHome(socketip)) {
+					// this is my domain, no need to send LRQs, just look into the endpoint table
+					PINDEX numberat = number.Find('@');	// always has an @
+					H225_ArrayOf_AliasAddress find_aliases;
+					find_aliases.SetSize(1);
+					PString local_alias = number.Mid(5,numberat-5);
+					H323SetAliasAddress(local_alias, find_aliases[0]);
+					endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
+					if (ep) {
+						// endpoint found locally
+						Route * route = new Route(ep);
+						route->m_policy = m_name;
+						return route;
+					}
+				} else {
+					PTRACE(3, "ROUTING\tSkipped, using only local LS for LRQs");
+				}
+			}
+		}
+	}
+	return NULL;
+}
+
+Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
+{
+	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
+		// only apply to urlID and h323ID
+		if ((aliases[i].GetTag() != H225_AliasAddress::e_url_ID)
+			&& (aliases[i].GetTag() != H225_AliasAddress::e_h323_ID))
+			continue;
+		PString alias(AsString(aliases[i], FALSE));
+		PINDEX at = alias.Find('@');
+		// skip empty aliases or those without at-sign
+	    if ((alias.GetLength() == 0) || (at == P_MAX_INDEX))
+			continue;
+
+		// DNS SRV Record lookup
+		PString number = "h323:" + alias;
+		PString domain = alias.Mid(at+1);
+		if (IsIPAddress(domain)
+			|| (domain.FindRegEx(PRegularExpression(":[0-9]+$", PRegularExpression::Extended)) != P_MAX_INDEX))
+			continue;	// don't use SRV record if domain part is IP or has port (Annex O, O.9), let dns policy handle them
+
+		// CS SRV Lookup
+		PStringList cs;
+		if (PDNS::LookupSRV(number, "_h323cs._tcp.", cs)) {
+			for (PINDEX j = 0; j < cs.GetSize(); j++) {
+				H225_TransportAddress dest;
+				PINDEX in = cs[j].Find('@');
+				PString dom = cs[j].Mid(in+1);
+				if (dom.Left(7) == "0.0.0.0") {
+					PTRACE(1, "ROUTING\tERROR in CS SRV lookup (" << cs[j] << ")");
+					continue;
+				}
+				PTRACE(4, "ROUTING\tSRV CS converted remote party " << alias << " to " << cs[j]);
+				if (GetTransportAddress(dom, GK_DEF_ENDPOINT_SIGNAL_PORT, dest)) {
+					PIPSocket::Address addr;
+					if (!(GetIPFromTransportAddr(dest, addr) && addr.IsValid()))
+						continue;
+					Route * route = NULL;
+					if (Toolkit::Instance()->IsGKHome(addr)) {
+						H225_ArrayOf_AliasAddress find_aliases;
+						find_aliases.SetSize(1);
+						H323SetAliasAddress(alias.Left(at), find_aliases[0]);
+						endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
+						if (ep) {
+							dest = ep->GetCallSignalAddress();
+							route = new Route(m_name, dest);
+							route->m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(dest);
+							return route;
+						} else {
+							return NULL;	// endpoint not found
+						}
+					}
+					if (!localonly) {
+						route = new Route(m_name, dest);
+						route->m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(dest);
+						return route;
+					}
+				}
+			}
+		}
+	}
+	return NULL;
+}
+
+// used for ARQs and Setups
+bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
+{
+	Route * route = CSLookup(aliases, false);
+	if (route) {
+		if (route->m_flags & Route::e_Reject) {
+			request.SetFlag(RoutingRequest::e_Reject);
+		} else {
+			request.AddRoute(*route);
+		}
+		delete route;
+		return true;
+	}
+	route = LSLookup(request, aliases);
+	if (route) {
+		if (route->m_flags & Route::e_Reject) {
+			request.SetFlag(RoutingRequest::e_Reject);
+		} else {
+			request.AddRoute(*route);
+		}
+		delete route;
+		return true;
+	}
+	return false;  
+}
+
+// usd for LRQs
 bool SRVPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
 { 
-    PTRACE(4, "ROUTING\tPolicy SRV not supported for LRQ");
+	Route * route = CSLookup(aliases, true);
+	if (route) {
+		if (route->m_flags & Route::e_Reject) {
+			request.SetFlag(RoutingRequest::e_Reject);
+		} else {
+			request.AddRoute(*route);
+		}
+		delete route;
+		return true;
+	}
+	route = LSLocalLookup(aliases);
+	if (route) {
+		if (route->m_flags & Route::e_Reject) {
+			request.SetFlag(RoutingRequest::e_Reject);
+		} else {
+			request.AddRoute(*route);
+		}
+		delete route;
+		return true;
+	}
 	return false;  
 }
 #endif
@@ -1541,9 +1733,6 @@ protected:
 
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
-
-private:
-	int m_Timeout;
 };
 
 bool RDSPolicy::FindByAliases(
@@ -1553,24 +1742,25 @@ bool RDSPolicy::FindByAliases(
 {
 	for (PINDEX a = 0; a < aliases.GetSize(); ++a) {
 		PString alias(AsString(aliases[a], FALSE));
-	    if (alias.GetLength() == 0) continue;
+	    if (alias.GetLength() == 0)
+			continue;
 
-	// DNS RDS Record lookup
+		// DNS RDS Record lookup
 		PString number;
 		PString domain;
 		PINDEX at = alias.Find('@');
 		if (at == P_MAX_INDEX) {
-		   number = "h323:t@" + alias;	
-		   domain = alias;
+			number = "h323:t@" + alias;	
+			domain = alias;
 	    } else {
-		   number = "h323:" + alias;
-		   domain = alias.Mid(at+1);
+			number = "h323:" + alias;
+			domain = alias.Mid(at+1);
 		}
 	
 		// LS Record lookup
 		PStringList ls;
-		 if (PDNS::RDSLookup(number,"H323+D2U",ls)) {
-			 for (PINDEX i=0; i<ls.GetSize(); i++) {
+		if (PDNS::RDSLookup(number,"H323+D2U",ls)) {
+			for (PINDEX i=0; i<ls.GetSize(); i++) {
 				PINDEX pos = ls[i].Find('@');
 				PString ipaddr = ls[i].Mid(pos + 1);
 				PTRACE(4, "ROUTING\tRDS LS located domain " << domain << " at " << ipaddr);
@@ -1585,31 +1775,31 @@ bool RDSPolicy::FindByAliases(
 
 				int m_neighborTimeout = GkConfig()->GetInteger(LRQFeaturesSection, "NeighborTimeout", 5) * 100;
 
-				// Send LRQ to retreive callers signalling address 
+				// Send LRQ to retreive callers signaling address 
 				LRQSender<AdmissionRequest> functor((AdmissionRequest &)request);
 				LRQRequester Request(functor);
 				if (Request.Send(nb)) {
 					if (H225_LocationConfirm *lcf = Request.WaitForDestination(m_neighborTimeout)) {
-							Route route(m_name, lcf->m_callSignalAddress);
-#ifdef hasH460
-			                if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
-			                    H225_RasMessage ras;
-			                    ras.SetTag(H225_RasMessage::e_locationConfirm);
-                                H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
-			                    con = *lcf;
-			                    route.m_destEndpoint = endptr(new EndpointRec(ras));	
-			                }
+						Route route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_H460
+						if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
+							H225_RasMessage ras;
+							ras.SetTag(H225_RasMessage::e_locationConfirm);
+							H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
+							con = *lcf;
+							route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);	
+						}
 #endif
-							request.AddRoute(route);
-							request.SetFlag(RoutingRequest::e_aliasesChanged);
-							return true;
-				    }
+						request.AddRoute(route);
+						request.SetFlag(RoutingRequest::e_aliasesChanged);
+						return true;
+					}
 				}
 				PTRACE(4, "ROUTING\tDNS RDS LRQ Error for " << domain << " at " << ipaddr);
-			 }
-			}  
-	} 
-  return false; 
+			}
+		}
+	}
+	return false;
 }
 
 
