@@ -9,7 +9,7 @@
 //
 // This work is published under the GNU Public License (GPL)
 // see file COPYING for details.
-// We also explicitely grant the right to link this code
+// We also explicitly grant the right to link this code
 // with the OpenH323 library.
 //
 // initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
@@ -18,7 +18,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASPDU_H
-#define RASPDU_H "@(#) $Id: RasPDU.h,v 1.11 2008/04/18 17:20:02 willamowius Exp $"
+#define RASPDU_H "@(#) $Id: RasPDU.h,v 1.15 2009/05/28 11:00:07 willamowius Exp $"
 
 #include <list>
 #include "yasocket.h"
@@ -56,7 +56,7 @@ struct GatekeeperMessage {
 class RasListener : public UDPSocket {
 public:
 	RasListener(const Address &, WORD);
-	~RasListener();
+	virtual ~RasListener();
 
 	GatekeeperMessage *ReadRas();
 	bool SendRas(const H225_RasMessage &, const Address &, WORD);
@@ -111,9 +111,9 @@ public:
 
 	static void Initialize();
 	static RegistrationTable *GetEndpointTbl()
-	        { return EndpointTbl; };
+	        { return EndpointTbl; }
 	static CallTable *GetCallTbl()
-	        { return CallTbl; }; 
+	        { return CallTbl; }
 protected:
 	RasMsg(GatekeeperMessage *m) : m_msg(m) {}
 	RasMsg(const RasMsg &);
@@ -137,6 +137,7 @@ public:
 	typedef RAS RasClass;
 	
 	RasPDU(GatekeeperMessage *m) : RasMsg(m), request(m->m_recvRAS) {}
+	virtual ~RasPDU() {}
 
 	// override from class RasMsg
 	virtual bool Process() { return false; }
@@ -262,7 +263,7 @@ public:
 	// longer lifetime than this object
 	RasRequester(H225_RasMessage &);
 	RasRequester(H225_RasMessage &, const Address &);
-	~RasRequester();
+	virtual ~RasRequester();
 
 	WORD GetSeqNum() const { return m_seqNum; }
 	bool WaitForResponse(int);
@@ -303,7 +304,7 @@ public:
 	typedef typename RasInfo<RAS>::ConfirmTag ConfirmTag;
 	typedef typename RasInfo<RAS>::RejectTag RejectTag;
 	Requester(H225_RasMessage &, const Address &);
-	~Requester() { this->m_rasSrv->UnregisterHandler(this); } // fix for GCC 3.4.2
+	virtual ~Requester() { this->m_rasSrv->UnregisterHandler(this); } // fix for GCC 3.4.2
 };
 
 template<class RAS>
