@@ -17,7 +17,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASSRV_H
-#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.51 2009/02/19 14:51:33 willamowius Exp $"
+#define RASSRV_H "@(#) $Id: RasSrv.h,v 1.55 2010/05/03 12:37:50 shorne Exp $"
 
 #include <vector>
 #include <list>
@@ -101,6 +101,7 @@ public:
 	WORD GetRequestSeqNum();
 
 	GkInterface *SelectInterface(const Address &);
+    GkInterface *SelectDefaultInterface();
 	const GkInterface *SelectInterface(const Address &) const;
 	RasListener * GetRasListener(const Address & addr) const;
 	Address GetLocalAddress(const Address &) const;
@@ -126,7 +127,7 @@ public:
 	const NeighborList *GetNeighbors() const { return neighbors; }
 	const VirtualQueue *GetVirtualQueue() const { return vqueue; }
 
-	// get signalling handler
+	// get signaling handler
 	ProxyHandler *GetSigProxyHandler();
 	ProxyHandler *GetRtpProxyHandler();
 
@@ -169,7 +170,12 @@ public:
 		callptr& call,
 		time_t now = 0
 		);
-	
+
+	bool LogAcctEvent(
+		int evt,
+		const endptr& ep
+		);
+
 	PString GetAuthInfo(
 		const PString &moduleName
 		);
@@ -207,7 +213,6 @@ public:
         H225_ArrayOf_AlternateGK assignedGK;
 		assignedGK.SetSize(0);
 
-		// Put query in here....
 		// Queries a DB for gatekeeper registrations with the first being
 		// the assigned gatekeeper. The alternates are then in preference order
 		// Note: If an assigned gatekeeper is found then the registration is not
