@@ -10,7 +10,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.174 2010/05/03 12:38:33 shorne Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.174.2.1 2010/05/26 13:02:44 willamowius Exp $"
 
 #include <list>
 #include <map>
@@ -1278,7 +1278,7 @@ private:
 	};
 
 	std::map<WORD,H46024Balternate> m_H46024Balternate;
-	void BuildH46024AnnexBMessage(bool initiate,H245_MultimediaSystemControlMessage & h245msg, const map<WORD,H46024Balternate> & alt);
+	void BuildH46024AnnexBMessage(bool initiate,H245_MultimediaSystemControlMessage & h245msg, const std::map<WORD,H46024Balternate> & alt);
 	list<int> m_h46024Bflag;
 	PMutex m_H46024Bmutex;
 #endif
