@@ -17,7 +17,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.35 2009/05/28 11:00:07 willamowius Exp $"
+#define ROUTING_H "@(#) $Id: Routing.h,v 1.39 2010/04/06 07:05:33 willamowius Exp $"
 
 #include <map>
 #include <list>
@@ -47,7 +47,8 @@ struct Route {
 	// a policy can set flags to indicate extra status of a processed request
 	enum Flags {
 		e_toParent = 1,
-		e_toNeighbor = 2
+		e_toNeighbor = 2,
+		e_Reject = 4
 	};
 	
 	Route();
@@ -112,6 +113,8 @@ public:
 	void SetFlag(unsigned f) { m_flags |= f; }
 	unsigned GetRejectReason() const { return m_reason; }
 	unsigned GetFlags() const { return m_flags; }
+	void SetSourceIP(const PString & ip) { m_sourceIP = ip; }
+	PString GetSourceIP() const { return m_sourceIP; }
 
 private:
 	RoutingRequest(const RoutingRequest&);
@@ -122,6 +125,7 @@ private:
 	unsigned m_flags; /// request specific flags
 	std::list<Route> m_routes;
 	std::list<Route> m_failedRoutes;
+	PString m_sourceIP;
 };
 
 template<class R, class W>
@@ -131,8 +135,8 @@ public:
 	typedef W Wrapper;
 
 	Request(ReqObj & r, Wrapper *w) : m_request(r), m_wrapper(w) {}
-	Request(ReqObj & r, Wrapper *w, const PString & id)
-		: m_request(r), m_wrapper(w), m_callingStationId(id) {}
+	Request(ReqObj & r, Wrapper *w, const PString & id, PUInt64 authid)
+		: m_request(r), m_wrapper(w), m_callingStationId(id), m_clientAuthId(authid) {}
 	Request(ReqObj & r, Wrapper *w, const std::list<Route> &failedRoutes)
 		: RoutingRequest(failedRoutes), m_request(r), m_wrapper(w) {}
 
@@ -145,6 +149,7 @@ public:
 	const ReqObj & GetRequest() const { return m_request; }
 	const Wrapper *GetWrapper() const { return m_wrapper; }
 	PString GetCallingStationId() const { return m_callingStationId; }
+	PUInt64 GetClientAuthId() const { return m_clientAuthId; }
 	const H225_ArrayOf_AliasAddress *GetAliases() const
 	{ return const_cast<Request<R, W> *>(this)->GetAliases(); }
 
@@ -152,6 +157,8 @@ private:
 	ReqObj & m_request;
 	Wrapper *m_wrapper;
 	PString m_callingStationId;
+	/// ID provided by client during authentication
+	PUInt64 m_clientAuthId;
 };
 
 typedef Request<H225_AdmissionRequest, RasMsg> AdmissionRequest;
@@ -302,6 +309,8 @@ public:
 		H225_ArrayOf_AliasAddress* destinationInfo,
 		/// destinationCallSignalAddr (optionally set by this function on successful return)
 		PString* callSigAdr,
+		/// bind IP for BindAndRouteToGateway
+		PString* bindIP,
 		/// should the call be rejected modified by this function on return)
 		bool & reject,
 		/// an actual virtual queue name (should be present in destinationInfo too)
@@ -334,6 +343,8 @@ public:
 		unsigned crv,
 		/// callID of the call associated with the route request
 		const PString& callID,
+		// outgoing IP or empty
+		const PString& bindIP = PString::Empty(),
 		/// should this call be rejected
 		bool reject = false
 		);
@@ -356,6 +367,8 @@ public:
 		unsigned crv,
 		/// callID of the call associated with the route request
 		const PString& callID,
+		// outgoing IP or empty
+		const PString& bindIP = PString::Empty(),
 		/// should this call be rejected
 		bool reject = false
 		);
@@ -390,11 +403,12 @@ private:
 			unsigned crv,
 			const PString& callID,
 			H225_ArrayOf_AliasAddress* agent,
-			PString* callsignaladdr
+			PString* callsignaladdr,
+			PString* bindIP
 			)
 			:
 			m_callingEpId((const char*)callingEpId), m_crv(crv), m_callID(callID),
-			m_agent(agent), m_callsignaladdr(callsignaladdr) {}
+			m_agent(agent), m_callsignaladdr(callsignaladdr), m_sourceIP(bindIP) {}
 
 		/// identifier for the endpoint associated with this request
 		PString m_callingEpId;
@@ -407,9 +421,11 @@ private:
 		H225_ArrayOf_AliasAddress* m_agent;
 		/// destinationCallSignallAddress for the target agent - target route IF NOT NULL
 		PString* m_callsignaladdr;
+		/// bindIP or empty
+		PString* m_sourceIP;
 		/// should this call be rejected
 		bool m_reject;
-		/// a synchronization point for signalling that routing decision
+		/// a synchronization point for signaling that routing decision
 		/// has been made by the external application
 		PSyncPoint m_sync;
 	};
@@ -429,6 +445,8 @@ private:
 		/// a pointer to a string to be filled with a callSignalAddress
 		/// when the routing decision has been made (optional)
 		PString* callSigAdr,
+		/// bind IP for BindAndRouteToGateway
+		PString* bindIP,
 		/// set by the function to true if another route request for the same
 		/// call is pending
 		bool& duplicate
