@@ -50,7 +50,7 @@ void SoftPBX::PrintEndpoint(const PString & EpStr, USocket *client, bool verbose
 	if (ep)
 		msg = "RCF|" + ep->PrintOn(verbose) + ";\r\n";
 	else
-		msg = "SoftPBX: endpoint " + EpStr + " not found!\r\n";
+		msg = "Endpoint " + EpStr + " not found!\r\n";
 	client->TransmitData(msg);
 }
 
@@ -87,6 +87,13 @@ void SoftPBX::PrintStatistics(USocket *client, bool)
 	client->TransmitData(msg);
 }
 
+void SoftPBX::ResetCallCounters(USocket *client)
+{
+	PTRACE(3, "GK\tSoftPBX: ResetCallCounters");
+    CallTable::Instance()->ResetCallCounters();
+	client->TransmitData("Call counters reset\r\n");
+}
+
 // send URQ to all endpoints
 void SoftPBX::UnregisterAllEndpoints()
 {
@@ -103,8 +110,8 @@ void SoftPBX::UnregisterAlias(PString Alias)
 
 	const endptr ep = RegistrationTable::Instance()->FindByAliases(EpAlias);
 	if (!ep) {
-		PString msg("SoftPBX: alias " + Alias + " not found!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("Alias " + Alias + " not found!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
@@ -114,8 +121,8 @@ void SoftPBX::UnregisterAlias(PString Alias)
 	// remove the endpoint (even if we don't get a UCF - the endoint might be dead)
 	RegistrationTable::Instance()->RemoveByEndptr(ep);
 
-	PString msg("SoftPBX: Endpoint " + Alias + " unregistered!");
-	PTRACE(2, "GK\t" + msg);
+	PString msg("Endpoint " + Alias + " unregistered!");
+	PTRACE(2, "GK\tSoftPBX: " + msg);
 	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
@@ -124,12 +131,16 @@ void SoftPBX::UnregisterIp(PString Ip)
 	H225_TransportAddress callSignalAddress;
 	GetTransportAddress(Ip, (WORD)GkConfig()->GetInteger("EndpointSignalPort", GK_DEF_ENDPOINT_SIGNAL_PORT), callSignalAddress);
 
-	PTRACE(3, "GK\tSoftPBX: UnregisterIp " << AsDotString(callSignalAddress));
+	PTRACE(3, "GK\tSoftPBX: UnregisterIp " << Ip);
 
-	const endptr ep = RegistrationTable::Instance()->FindBySignalAdr(callSignalAddress);
+	endptr ep;
+	if (Ip.Find(':') == P_MAX_INDEX)
+		ep = RegistrationTable::Instance()->FindBySignalAdrIgnorePort(callSignalAddress);	// no port specified
+	else
+		ep = RegistrationTable::Instance()->FindBySignalAdr(callSignalAddress);	// port specify, search for it
 	if (!ep) {
-		PString msg("SoftPBX: ip " + AsDotString(callSignalAddress) + " not found!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("IP " + Ip + " not found!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
@@ -139,8 +150,8 @@ void SoftPBX::UnregisterIp(PString Ip)
 	// remove the endpoint (even if we don't get a UCF - the endoint might be dead)
 	RegistrationTable::Instance()->RemoveByEndptr(ep);
 
-	PString msg("SoftPBX: Endpoint " + AsDotString(callSignalAddress) + " unregistered!");
-	PTRACE(2, "GK\t" + msg);
+	PString msg("Endpoint " + AsDotString(callSignalAddress) + " unregistered!");
+	PTRACE(2, "GK\tSoftPBX: " + msg);
 	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
@@ -164,7 +175,7 @@ void SoftPBX::DisconnectCall(unsigned CallNumber)
 		return;
 	}
 
-	Call->Disconnect();
+	Call->Disconnect(true);
 	// remove the call directly so we don't have to handle DCF
 	CallTable::Instance()->RemoveCall(Call);
 
@@ -178,9 +189,28 @@ void SoftPBX::DisconnectIp(PString Ip)
 {
 	H225_TransportAddress callSignalAddress;
 	GetTransportAddress(Ip, (WORD)GkConfig()->GetInteger("EndpointSignalPort", GK_DEF_ENDPOINT_SIGNAL_PORT), callSignalAddress);
-	PTRACE(3, "GK\tSoftPBX: DisconnectIp " << AsDotString(callSignalAddress));
+	PTRACE(3, "GK\tSoftPBX: DisconnectIp " << Ip);
 
-	DisconnectEndpoint(RegistrationTable::Instance()->FindBySignalAdr(callSignalAddress));
+	callptr Call;
+	if (Ip.Find(':') == P_MAX_INDEX)
+		Call = CallTable::Instance()->FindBySignalAdrIgnorePort(callSignalAddress);	// no port specified
+	else
+		Call = CallTable::Instance()->FindBySignalAdr(callSignalAddress);	// port specify, search for it
+	if (!Call) {
+		PString msg = "Can't find call for IP " + Ip;
+		PTRACE(2, "GK\tSoftPBX: " << msg);
+		GkStatus::Instance()->SignalStatus(msg + "\r\n");
+		return;
+	}
+	unsigned CallNumber = Call->GetCallNumber();
+
+	Call->Disconnect(true);
+	// remove the call directly so we don't have to handle DCF
+	CallTable::Instance()->RemoveCall(Call);
+
+	PString msg(PString::Printf, "Call number %d disconnected.", CallNumber);
+	PTRACE(2, "GK\tSoftPBX: " << msg);
+	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
 // send a DRQ to this endpoint
@@ -191,6 +221,7 @@ void SoftPBX::DisconnectAlias(PString Alias)
 	H323SetAliasAddress(Alias, EpAlias[0]);
 	PTRACE(3, "GK\tSoftPBX: DisconnectAlias " << Alias);
 
+	// TODO: extend for unregistered endpoints
 	DisconnectEndpoint(RegistrationTable::Instance()->FindByAliases(EpAlias));
 }
 
@@ -207,8 +238,8 @@ void SoftPBX::DisconnectEndpoint(PString Id)
 void SoftPBX::DisconnectEndpoint(const endptr &ep)
 {
 	if (!ep) {
-		PString msg("SoftPBX: no endpoint to disconnect!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("No endpoint to disconnect!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
@@ -239,14 +270,14 @@ void SoftPBX::SendProceeding(PString CallId)
 	if (call) {
 		lForwardedSocket = call->GetCallSignalSocketCalling();
 		if (!lForwardedSocket) {
-			PString msg("SoftPBX: can't find signalling socket (direct mode ?)");
-			PTRACE(1, "GK\t" + msg);
+			PString msg("Can't find signaling socket (direct mode ?)");
+			PTRACE(1, "GK\tSoftPBX: " + msg);
 			GkStatus::Instance()->SignalStatus(msg + "\r\n");
 			return;
 		}
 	} else {
-		PString msg("SoftPBX: no call to send CallProceeding! " + AsString(cid.m_guid));
-		PTRACE(1, "GK\t" + msg);
+		PString msg("No call to send CallProceeding! " + AsString(cid.m_guid));
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
@@ -261,8 +292,8 @@ void SoftPBX::SendProceeding(PString CallId)
 bool SoftPBX::TransferCall(endptr & lSrcForward, SmartPtr<CallRec> lCall, PString DestinationAlias)
 {
 	if (!lCall || !lSrcForward) {
-		PString msg("SoftPBX: no call to transfer!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("No call to transfer!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		return false;
 	}
 
@@ -277,8 +308,8 @@ bool SoftPBX::TransferCall(endptr & lSrcForward, SmartPtr<CallRec> lCall, PStrin
 		lForwardedSocket = lCall->GetCallSignalSocketCalled();
 	}
 	if (!lForwardedSocket) {
-		PString msg("SoftPBX: can't transfer call in direct mode!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("Can't transfer call in direct mode!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return false;
 	}
@@ -294,8 +325,8 @@ bool SoftPBX::TransferCall(endptr & lSrcForward, SmartPtr<CallRec> lCall, PStrin
 	lBufferAliasArray.RemoveAll();
 
 	if (!lDestForward) {
-		PString msg("SoftPBX: transferred destination not found!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("Transferred destination not found!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		return false;
 	}
 
@@ -308,8 +339,8 @@ bool SoftPBX::TransferCall(endptr & lSrcForward, SmartPtr<CallRec> lCall, PStrin
 	q931.Encode(lBuffer);
 	lForwardedSocket->TransmitData(lBuffer);
 
-	PString msg = PString("SoftPBX: call ") + PString(lCall->GetCallNumber()) + " transfer success.";
-	PTRACE(1, "GK\t" + msg);
+	PString msg = PString("Call ") + PString(lCall->GetCallNumber()) + " transfer success.";
+	PTRACE(1, "GK\tSoftPBX: " + msg);
  
 	return true;
 
@@ -336,8 +367,8 @@ void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
 	lBufferAliasArray.RemoveAll();
 
 	if (!lSrcForward) {
-		PString msg("SoftPBX: no endpoint to transfer!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("No endpoint to transfer!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
@@ -355,14 +386,14 @@ void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
 			lForwardedSocket = lCall->GetCallSignalSocketCalled();
 		}
 		if (!lForwardedSocket) {
-			PString msg("SoftPBX: can't transfer call in direct mode!");
-			PTRACE(1, "GK\t" + msg);
+			PString msg("Can't transfer call in direct mode!");
+			PTRACE(1, "GK\tSoftPBX: " + msg);
 			GkStatus::Instance()->SignalStatus(msg + "\r\n");
 			return;
 		}
 	} else {
-		PString msg("SoftPBX: no call to transfer!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("No call to transfer!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
@@ -376,8 +407,8 @@ void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
 	lBufferAliasArray.RemoveAll();
 
 	if (!lDestForward) {
-		PString msg("SoftPBX: transferred destination not found!");
-		PTRACE(1, "GK\t" + msg);
+		PString msg("Transfer destination not found!");
+		PTRACE(1, "GK\tSoftPBX: " + msg);
 		GkStatus::Instance()->SignalStatus(msg + "\r\n");
 		return;
 	}
@@ -391,8 +422,8 @@ void SoftPBX::TransferCall(PString SourceAlias, PString DestinationAlias)
 	q931.Encode(lBuffer);
 	lForwardedSocket->TransmitData(lBuffer);
 
-	PString msg = PString("SoftPBX: call ") + PString(lCall->GetCallNumber()) + " transferred from" + SourceAlias + " to " + DestinationAlias;
-	PTRACE(1, "GK\t" + msg);
+	PString msg = PString("Call ") + PString(lCall->GetCallNumber()) + " transferred from" + SourceAlias + " to " + DestinationAlias;
+	PTRACE(1, "GK\tSoftPBX: " + msg);
 	GkStatus::Instance()->SignalStatus(msg + "\r\n");
 }
 
