@@ -25,6 +25,8 @@
 #endif
 #include "clirw.h"
 #include "capctrl.h"
+#include "RasSrv.h"
+#include "RasTbl.h"
 #include "Toolkit.h"
 #include "gk_const.h"
 
@@ -95,7 +97,9 @@ unsigned NetworkAddress::GetNetmaskLen() const
 	return sz - len;
 }
 
-bool NetworkAddress::operator==(const NetworkAddress &addr) const
+// TODO: check if this operator return true for unequal addresses
+// BUG: 9.9.9.0/24 == 9.9.10.0/24
+bool NetworkAddress::operator==(const NetworkAddress & addr) const
 {
 	if (m_address.GetSize() != addr.m_address.GetSize())
 		return false;
@@ -104,7 +108,7 @@ bool NetworkAddress::operator==(const NetworkAddress &addr) const
 	for (unsigned i = 0; i < sz; i++)
 		if (m_address[i] != addr.m_address[i] || m_netmask[i] != addr.m_netmask[i])
 			return false;
-			
+
 	return true;
 }
 
@@ -177,7 +181,7 @@ PString NetworkAddress::AsString() const
 
 bool NetworkAddress::IsAny() const
 {
-	return const_cast<NetworkAddress*>(this)->m_address.IsAny() ? true : false;
+	return const_cast<NetworkAddress*>(this)->m_address.IsAny();
 }
 
 bool NetworkAddress::operator<(const NetworkAddress &addr) const
@@ -248,7 +252,7 @@ inline bool Toolkit::RouteTable::RouteEntry::Compare(const Address *ip) const
 // class Toolkit::RouteTable
 void Toolkit::RouteTable::InitTable()
 {
-	// Workaround for OS doesn't support GetRouteTable
+	// workaround for OS that don't support GetRouteTable
 	PIPSocket::GetHostAddress(defAddr);
 
 	ClearTable();
@@ -280,10 +284,14 @@ void Toolkit::RouteTable::InitTable()
     }
 
 #if PTRACING
-	for (RouteEntry *entry = rtable_begin; entry != rtable_end; ++entry)
+	for (RouteEntry *entry = rtable_begin; entry != rtable_end; ++entry) {
 		PTRACE(2, "Network=" << entry->GetNetwork() << '/' << entry->GetNetMask() <<
 			  ", IP=" << entry->GetDestination());
+	}
 	PTRACE(2, "Default IP=" << defAddr);
+	if (defAddr.IsLoopback()) {
+		PTRACE(1, "WARNING: Your default IP=" << defAddr << " is a loopback address. That probably won't work!");
+	}
 #endif
 }
 
@@ -297,19 +305,50 @@ void Toolkit::RouteTable::ClearTable()
 	}
 }
 
-// can't pass a reference of Address, or STL complains...
+void Toolkit::RouteTable::AddInternalNetwork(const NetworkAddress & network)
+{
+	if (find(m_internalnetworks.begin(), m_internalnetworks.end(), network) == m_internalnetworks.end())
+		m_internalnetworks.push_back(network);
+}
+
 PIPSocket::Address Toolkit::RouteTable::GetLocalAddress(const Address & addr) const
 {
-  // If a dynamic external IP retrieve external IP from DNS entries
-	if (DynExtIP && !addr.IsRFC1918()) {
-	  PIPSocket::Address extip;
-	  H323TransportAddress ex = H323TransportAddress(ExtIP);
-	  ex.GetIpAddress(extip);
-	  return extip;
+	// look through internal networks and make sure we don't return the external IP for them
+	for (unsigned j = 0; j < m_internalnetworks.size(); ++j) {
+		if (addr << m_internalnetworks[j]) {
+			// check if internal network is in route table, but don't use the default route
+			RouteEntry *entry = find_if(rtable_begin, rtable_end,
+				bind2nd(mem_fun_ref(&RouteEntry::Compare), &addr));
+			if ((entry != rtable_end) && (entry->GetNetMask() != INADDR_ANY))
+				return entry->GetDestination();
+			else
+				return defAddr;
+		}
+	}
+
+	// if external IP is configured
+    if (!ExtIP) {
+      if (DynExtIP) {  // if dynamic resolve DNS entry
+        PIPSocket::Address extip;
+		H323TransportAddress ex = H323TransportAddress(ExtIP);
+		ex.GetIpAddress(extip);
+          if (extip.IsValid()) {
+              return extip;
+          } else {
+              PTRACE(2,"NAT\tERROR: ExtIP " << ExtIP << " unresolvable." );
+          }
+      } else {  // If valid IP then use the ExtIP value
+          PIPSocket::Address extip(ExtIP);
+          if (extip.IsValid()) {
+              return extip;
+          } else {
+              PTRACE(2,"NAT\tERROR: ExtIP " << ExtIP << " unuseable." );
+          }
+      }
 	}
 
 	RouteEntry *entry = find_if(rtable_begin, rtable_end,
-			bind2nd(mem_fun_ref(&RouteEntry::Compare), &addr));
+		bind2nd(mem_fun_ref(&RouteEntry::Compare), &addr));
 	return (entry != rtable_end) ? entry->GetDestination() : defAddr;
 }
 
@@ -343,9 +382,9 @@ bool Toolkit::RouteTable::CreateRouteTable(const PString & extroute)
 
 	rtable_end = rtable_begin = static_cast<RouteEntry *>(::malloc(i * sizeof(RouteEntry)));
 	for (PINDEX r = 0; r < i ; ++r) {
-		if (!extroute && (r==r_table.GetSize()))
+		if (!extroute && (r==r_table.GetSize())) {
 			::new (rtable_end++) RouteEntry(extroute);
-		else {
+		} else {
 		  PIPSocket::RouteEntry & r_entry = r_table[r];
 		  if (r_entry.GetNetMask() != INADDR_ANY) 
 			::new (rtable_end++) RouteEntry(r_entry, if_table);
@@ -359,14 +398,14 @@ bool Toolkit::VirtualRouteTable::CreateTable()
 {
 	PString nets = GkConfig()->GetString("NetworkInterfaces", "");
 	if (!nets) {		
-	   PStringArray networks(nets.Tokenise(" ,;\t", FALSE));
-	   int i = networks.GetSize();
-	   if (i > 0) {
-          rtable_end = rtable_begin = static_cast<RouteEntry *>(::malloc(i * sizeof(RouteEntry)));
-		  for (PINDEX r = 0; r < i ; ++r) 
-             ::new (rtable_end++) RouteEntry(networks[r]);
-       }
-	   return true;
+		PStringArray networks(nets.Tokenise(" ,;\t", FALSE));
+		int i = networks.GetSize();
+		if (i > 0) {
+			rtable_end = rtable_begin = static_cast<RouteEntry *>(::malloc(i * sizeof(RouteEntry)));
+			for (PINDEX r = 0; r < i ; ++r) {
+				::new (rtable_end++) RouteEntry(networks[r]);
+			}
+		}
 	}
 
 	// If we have an external IP setting then load the detected Route Table and add a route for the external IP
@@ -377,15 +416,15 @@ bool Toolkit::VirtualRouteTable::CreateTable()
 	PIPSocket::Address ext((DWORD)0);
 	H323TransportAddress ex = H323TransportAddress(extip);
 	ex.GetIpAddress(ext);
-	if (ext.IsValid() && !ext.IsRFC1918()) {
-	   ExtIP = extip;
-	   PString extroute = PString();
-   	   if (!DynExtIP) 
+	if (ext.IsValid()) {
+		ExtIP = extip;
+		PString extroute = PString();
+		if (!DynExtIP) 
 		  extroute = ext.AsString() + "/0";
 
-	   CreateRouteTable(extroute);
-	   PTRACE(1,"External IP = " << ExtIP << " dynamic " << DynExtIP);
-	   return true;
+		CreateRouteTable(extroute);
+		PTRACE(1,"External IP = " << ExtIP << " dynamic " << DynExtIP);
+		return true;
 	} else
         DynExtIP = false;
 
@@ -414,44 +453,157 @@ Toolkit::ProxyCriterion::~ProxyCriterion()
 
 void Toolkit::ProxyCriterion::LoadConfig(PConfig *config)
 {
+	// read switch for default proxy mode
 	m_enable = AsBool(config->GetString(ProxySection, "Enable", "0"));
 	if (!m_enable) {
-		PTRACE(2, "GK\tH.323 Proxy disabled");
-		return;
+		PTRACE(2, "GK\tH.323 Proxy not enabled by default");
+	} else {
+		PTRACE(2, "GK\tH.323 Proxy enabled");
 	}
 
-	PTRACE(2, "GK\tH.323 Proxy enabled");
-
-	m_networks.clear();
+	m_internalnetworks.clear();
+	m_modeselection.clear();
 
 	PStringArray networks(config->GetString(ProxySection, "InternalNetwork", "").Tokenise(" ,;\t", FALSE));
 
 	// if no networks specified then use the detected values
+	int signalRoutingMode = CallRec::Undefined;
+	if (RasServer::Instance()->IsGKRouted()) {
+		if (RasServer::Instance()->IsH245Routed()) {
+			signalRoutingMode = CallRec::H245Routed;
+		} else {
+			signalRoutingMode = CallRec::SignalRouted;
+		}
+	}
+	NetworkModes internal_netmode;
+	internal_netmode.fromExternal = m_enable ? CallRec::Proxied : signalRoutingMode;
+	internal_netmode.insideNetwork = signalRoutingMode;
 	if (networks.GetSize() == 0) {
-	  m_networks = Toolkit::Instance()->GetInternalNetworks();
-	  for (unsigned j = 0; j < m_networks.size(); ++j) 
-		PTRACE(2, "GK\tInternal Network " << j << " = " << m_networks[j].AsString());
-	  return;
-	} 
+		m_internalnetworks = Toolkit::Instance()->GetInternalNetworks();
+		for (unsigned j = 0; j < m_internalnetworks.size(); ++j) {
+			m_modeselection[m_internalnetworks[j]] = internal_netmode;
+			PTRACE(2, "GK\tInternal Network " << j << " = " << m_internalnetworks[j].AsString()
+					<< " (" << internal_netmode.fromExternal << "," << internal_netmode.insideNetwork << ")");
+		}
+	} else {
+		for (PINDEX i = 0; i < networks.GetSize(); ++i) {
+			m_internalnetworks.push_back(networks[i]);
+			Toolkit::Instance()->GetRouteTable()->AddInternalNetwork(networks[i]);
+			m_modeselection[networks[i]] = internal_netmode;
+			PTRACE(2, "GK\tINI Internal Network " << i << " = " << m_internalnetworks[i].AsString()
+					<< " (" << internal_netmode.fromExternal << "," << internal_netmode.insideNetwork << ")");
+		}
+	}
 
-	  for (PINDEX i = 0; i < networks.GetSize(); ++i) {
-		m_networks.resize(m_networks.size() + 1);
-		m_networks[m_networks.size() - 1] = NetworkAddress(networks[i]);
-		PTRACE(2, "GK\tINI Internal Network " << i << " = " << m_networks.back().AsString());
-	  }
+	// read [ModeSelection] section
+	PStringToString mode_rules(config->GetAllKeyValues("ModeSelection"));
+	for (PINDEX i = 0; i < mode_rules.GetSize(); ++i) {
+		PString network = mode_rules.GetKeyAt(i);
+		if (!network.IsEmpty()) {
+			PStringArray modes((mode_rules.GetDataAt(i)).Tokenise(" ,;\t", FALSE));
+			if (modes.GetSize() >= 1 && modes.GetSize() <= 2) {
+				NetworkAddress addr = NetworkAddress(network);
+				NetworkModes netmode;
+				netmode.fromExternal = ToRoutingMode(modes[0].Trim());
+				netmode.insideNetwork = netmode.fromExternal;
+				if (modes.GetSize() == 2)
+					netmode.insideNetwork = ToRoutingMode(modes[1].Trim());
+				// check if we have one or more (implied) rules for internal networks that we have to delete first
+				std::map<NetworkAddress, NetworkModes>::iterator iter = m_modeselection.begin();
+				while (iter != m_modeselection.end()) {
+					if (iter->first << addr) {
+						PTRACE(2, "Removing (implied) ModeSelection rule for " << iter->first.AsString());
+						m_modeselection.erase(iter++);
+					} else {
+						iter++;
+					}
+				}
+				m_modeselection[addr] = netmode;
+				PTRACE(2, "GK\tModeSelection rule: " << addr.AsString() << "=" << netmode.fromExternal << "," << netmode.insideNetwork);
+			} else {
+				PTRACE(1, "GK\tInvalid ModeSelection rule: " << mode_rules.GetKeyAt(i) << "=" << mode_rules.GetDataAt(i) );
+			}
+		}
+	}
 }
 
-bool Toolkit::ProxyCriterion::Required(const Address & ip1, const Address & ip2) const
+int Toolkit::ProxyCriterion::ToRoutingMode(const PCaselessString & mode) const
 {
-	return m_enable ? (m_networks.empty() || (IsInternal(ip1) != IsInternal(ip2))) : false;
+	if (mode == "Routed")
+		return CallRec::SignalRouted;
+	else if (mode == "H245Routed")
+		return CallRec::H245Routed;
+	else if (mode == "Proxy")
+		return CallRec::Proxied;
+	else
+		return CallRec::Undefined;	
+}
+
+// returns network for the rule or IsAny() when no is rule found
+NetworkAddress Toolkit::ProxyCriterion::FindModeRule(const NetworkAddress & ip) const
+{
+	NetworkAddress bestmatch;
+
+	std::map<NetworkAddress, NetworkModes>::const_iterator iter = m_modeselection.begin();
+	while (iter != m_modeselection.end()) {
+		if ((ip << iter->first) && (iter->first.GetNetmaskLen() >= bestmatch.GetNetmaskLen())) {
+			bestmatch = iter->first;
+		}
+		iter++;
+	}
+
+	return bestmatch;
+}
+
+int Toolkit::ProxyCriterion::SelectRoutingMode(const Address & ip1, const Address & ip2) const
+{
+	// default mode
+	int mode = m_enable ? CallRec::Proxied : CallRec::SignalRouted;
+	if (mode == CallRec::SignalRouted && RasServer::Instance()->IsH245Routed())
+		mode = CallRec::H245Routed;
+
+	// check if we have a more specific setting
+	NetworkAddress bestMatchIP1 = FindModeRule(ip1);
+	NetworkAddress bestMatchIP2 = FindModeRule(ip2);
+
+	std::map<NetworkAddress, NetworkModes>::const_iterator iter;
+	// check for same network
+	// TODO: add invalid state to NetworkAddress so we can have ModeSlection rules for 0.0.0.0/0 that set a global default
+	if (!bestMatchIP1.IsAny() && !bestMatchIP2.IsAny()) {
+		// rules for both IPs
+		if (bestMatchIP1.Compare(bestMatchIP2) == 0) {	// operator== broken ?
+			// both on same network
+			iter = m_modeselection.find(bestMatchIP1);
+			if (iter != m_modeselection.end()) {
+				mode = iter->second.insideNetwork;
+			}
+		} else {
+			// on different networks, use maximum poxying
+			iter = m_modeselection.find(bestMatchIP1);
+			int mode1 = iter->second.fromExternal;	// no check, must exist
+			iter = m_modeselection.find(bestMatchIP2);
+			int mode2 = iter->second.fromExternal;	// no check, must exist
+			mode = max(mode1, mode2);
+		}
+	} else {
+		// only one rule, use that
+		iter = m_modeselection.find(bestMatchIP1);
+		if (iter != m_modeselection.end())
+			mode = iter->second.fromExternal;
+		iter = m_modeselection.find(bestMatchIP2);
+		if (iter != m_modeselection.end())
+			mode = iter->second.fromExternal;
+	}
+
+	return mode;
 }
 
 int Toolkit::ProxyCriterion::IsInternal(const Address & ip) const
 {
    // Return the network Id. Addresses may be on different internal networks
 	int retval = 0;
-	std::vector<NetworkAddress>::const_iterator i = m_networks.begin();
-	while (i != m_networks.end()) {
+	std::vector<NetworkAddress>::const_iterator i = m_internalnetworks.begin();
+	while (i != m_internalnetworks.end()) {
 		retval++;
 		if (ip << *i++)
 			return retval;
@@ -528,12 +680,23 @@ void Toolkit::RewriteTool::LoadConfig(
 {
 	m_RewriteFastmatch = config->GetString(RewriteSection, "Fastmatch", "");
 	m_TrailingChar = config->GetString("RasSrv::ARQFeatures", "RemoveTrailingChar", " ")[0];
-	m_defaultDomain = config->GetString("Gatekeeper::Main", "DefaultDomain", "");
+	PString defDomain = config->GetString("Gatekeeper::Main", "DefaultDomain", "");
+	m_defaultDomain = defDomain.Tokenise(",");
 	delete m_Rewrite;
 	m_Rewrite = new RewriteData(config, RewriteSection);
 	m_Rewrite->AddSection(config,AliasRewriteSection);
 }
 
+bool Toolkit::IsNumeric(const PString & s) 
+{
+	 PINDEX j;
+     for (j = 0; j < s.GetLength(); ++j)
+	       if (!isdigit(static_cast<unsigned char>(s[j])))
+	   	         break;
+
+	 return (j >= s.GetLength());
+}
+
 bool Toolkit::RewriteTool::RewritePString(PString & s) const
 {
 	bool changed = false;
@@ -542,25 +705,24 @@ bool Toolkit::RewriteTool::RewritePString(PString & s) const
 	 PINDEX at = s.Find('@');
 	 if (at != P_MAX_INDEX) {
 		 PString num = s.Left(at);
+		 PString domain = s.Mid(at+1);
+		 PIPSocket::Address domIP(domain);
+
          // Check if we have a default domain and strip it
-		 if (s.Mid(at+1) == m_defaultDomain) {
-		   PTRACE(2, "\tRewriteDomain: " << s << " to " << num);
+		 for (PINDEX i=0; i < m_defaultDomain.GetSize(); i++) {
+			 if (domain == m_defaultDomain[i]) {
+			   PTRACE(2, "\tRewriteDomain: " << s << " to " << num);
+			   s = num;
+			   changed = true;
+			   break;
+			 }
+		 }
+
+	     // Check that the domain is not a local IP address.
+		 if (!changed && domIP.IsValid() && Toolkit::Instance()->IsGKHome(domIP)) {
+ 		   PTRACE(2, "\tRemoveDomain: " << domain << " to " << num);
 		   s = num;
 		   changed = true;
-		 } else {
-			 // Check if all numeric then is E164 then strip the domain
-			 // this is used with SRV policy which includes the domain when 
-			 // sending the LRQ. 
-			 PINDEX j;
- 		     for (j = 0; j < num.GetLength(); ++j)
-			       if (!isdigit(static_cast<unsigned char>(num[j])))
-			   	         break;
-
-			 if (j >= num.GetLength()) { // is numeric
-				PTRACE(2, "\tRewriteToE164: " << s << " to " << num);
-                s = num;
-				changed = true;
-			 }
 		 }
 	 }
 
@@ -1085,7 +1247,7 @@ void Toolkit::ReloadSQLConfig()
 		queryResult = NULL;
 	}
 
-// Assigned Alias Query
+	// Assigned Alias Query
 	query = m_Config->GetString("SQLConfig", "AssignedAliasQuery", "");
 	if (!query.IsEmpty()) {
 		PTRACE(4, "SQLCONF\tLoading Assigned Alias rules from SQL database");
@@ -1302,6 +1464,10 @@ PConfig* Toolkit::ReloadConfig()
 #ifdef HAS_H46018
 	m_H46018Enabled	= m_Config->GetBoolean(RoutedSec, "EnableH46018", 0);
 #endif
+#ifdef HAS_H46023
+	m_H46023Enabled	= (m_Config->GetBoolean(RoutedSec, "EnableH46023", 0) && 
+						!m_Config->GetString(RoutedSec, "H46023STUN",""));
+#endif
 
 	// TODO/BUG: always call SetGKHome() on reload, even if we don't have a Home= setting
 	// otherwise we won't detect new IPs on the machine
@@ -1312,6 +1478,12 @@ PConfig* Toolkit::ReloadConfig()
 	m_RouteTable.InitTable();
 	m_VirtualRouteTable.InitTable();
 	m_ProxyCriterion.LoadConfig(m_Config);
+#ifdef HAS_H46023
+	if (m_H46023Enabled)  LoadH46023STUN();
+#endif
+#ifdef HAS_H460P
+	m_presence.LoadConfig(m_Config);
+#endif
 	m_Rewrite.LoadConfig(m_Config);
 	m_GWRewrite.LoadConfig(m_Config);
 	m_AssignedEPAliases.LoadConfig(m_Config);
@@ -1366,9 +1538,7 @@ void Toolkit::LoadCauseMap(
 }
 
 // load H.225 reason to Q.931 cause mapping
-void Toolkit::LoadReasonMap(
-	PConfig *cfg
-	)
+void Toolkit::LoadReasonMap(PConfig *cfg)
 {
 	// default to ITU-T Recommendation H.225
 	unsigned DefaultH225ReasonToQ931Cause[] =	{
@@ -1395,7 +1565,7 @@ bool Toolkit::MatchRegex(const PString &str, const PString &regexStr)
 		return FALSE;
 	}
 	if(!regex.Execute(str, pos)) {
-		PTRACE(5, "Gk\tRegex '"<<regexStr<<"' did not match '"<<str<<"'");
+		// PTRACE(6, "Gk\tRegex '" << regexStr << "' did not match '" << str << "'");
 		return FALSE;
 	}
 	return TRUE;
@@ -1823,11 +1993,11 @@ void Toolkit::AssignedGatekeepers::LoadConfig(PConfig * m_config)
 bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias,const PIPSocket::Address & ip, PStringArray & addresses)
 {
 
-// Support Gatekeeper discovery
+	// Support Gatekeeper discovery
     if (!GkConfig()->GetString(H350Section, "GatekeeperDiscovery", "0"))
 		   return false;
 
-// Search the Directory
+	// Search the Directory
 	PString search = GkConfig()->GetString(H350Section, "SearchBaseDN", "");
 
 	H225_AliasAddress aliasaddress;
@@ -1859,7 +2029,7 @@ bool Toolkit::AssignedGatekeepers::QueryH350Directory(const PString & alias,cons
 	   return false;
 	}
 
-// Locate the record
+	// Locate the record
 	for (H350_Session::LDAP_RecordList::const_iterator x = rec.begin(); x != rec.end(); ++x) {			
        H350_Session::LDAP_Record entry = x->second;
 	   PString gk;
@@ -2082,18 +2252,20 @@ bool Toolkit::RewriteE164(H225_AliasAddress & alias)
 
 	bool changed = RewritePString(E164);
 	if (changed) {
-		if (alias.GetTag() == H225_AliasAddress::e_dialedDigits)
-			H323SetAliasAddress(E164, alias, alias.GetTag());
+		if (E164.Find("@") != P_MAX_INDEX)
+			H323SetAliasAddress(E164, alias,H225_AliasAddress::e_url_ID);
+		else if (IsNumeric(E164))
+			H323SetAliasAddress(E164, alias, H225_AliasAddress::e_dialedDigits);
+		else if (alias.GetTag() != H225_AliasAddress::e_partyNumber) 
+			H323SetAliasAddress(E164, alias, H225_AliasAddress::e_h323_ID);
 		else {
-			if (alias.GetTag() == H225_AliasAddress::e_partyNumber) {
-				H225_PartyNumber & partyNumber = alias;
-				if (partyNumber.GetTag() == H225_PartyNumber::e_e164Number) {
-					H225_PublicPartyNumber &number = partyNumber;
-					number.m_publicNumberDigits = E164;
-				} else if (partyNumber.GetTag() == H225_PartyNumber::e_privateNumber) {
-					H225_PrivatePartyNumber &number = partyNumber;
-					number.m_privateNumberDigits = E164;
-				}
+			H225_PartyNumber & partyNumber = alias;
+			if (partyNumber.GetTag() == H225_PartyNumber::e_e164Number) {
+				H225_PublicPartyNumber &number = partyNumber;
+				number.m_publicNumberDigits = E164;
+			} else if (partyNumber.GetTag() == H225_PartyNumber::e_privateNumber) {
+				H225_PrivatePartyNumber &number = partyNumber;
+				number.m_privateNumberDigits = E164;
 			}
 		}
 	}
@@ -2161,6 +2333,71 @@ bool Toolkit::isBehindNAT(PIPSocket::Address & externalIP) const {
    return (m_VirtualRouteTable.IsMasquerade(externalIP));
 }
 
+#ifdef HAS_H46023
+bool Toolkit::IsH46023Enabled() const 
+{ 
+	return (m_H46023Enabled && (             // is enabled and
+#ifdef HAS_H46018
+		   m_H46018Enabled ||              // used with H.460.18 or
+#endif
+		   m_Config->GetBoolean(RoutedSec, "SupportCallingNATedEndpoints",1))); // GnuGk Native NAT Support
+}
+
+void Toolkit::LoadH46023STUN()
+{
+	m_H46023STUN.clear();
+
+	PString stun = m_Config->GetString(RoutedSec, "H46023STUN", "");
+	PStringArray stunlist = stun.Tokenise(",");
+
+	for (PINDEX i = 0; i < stunlist.GetSize(); i++) {
+	    PIPSocket::Address ip; WORD port;
+		GetTransportAddress(stunlist[i], GK_DEF_STUN_PORT, ip, port);
+		if (ip.IsValid()) {
+		     int intID = m_ProxyCriterion.IsInternal(ip);
+			 std::map<int,H323TransportAddress>::const_iterator inf = m_H46023STUN.find(intID);
+			 if (inf == m_H46023STUN.end()) {
+				 if (intID > 0) {
+				    PTRACE(4,"Std23\tSTUN Internal " << ip << " IF " << intID-1);
+				 } else {
+					PTRACE(4,"Std23\tSTUN Public Server " << stunlist[i] << " (" << ip << ")");
+				 }
+				m_H46023STUN.insert(pair<int, H323TransportAddress>(intID, H323TransportAddress(ip,port)));
+			 }
+		}
+	}
+}
+
+bool Toolkit::GetH46023STUN(const PIPSocket::Address & addr, H323TransportAddress & stun)
+{
+	 int intID = m_ProxyCriterion.IsInternal(addr);
+	 std::map<int,H323TransportAddress>::const_iterator inf = m_H46023STUN.find(intID);
+	 if (inf != m_H46023STUN.end()) {
+		 stun = inf->second;
+		 return true;
+	 }
+	PTRACE(2,"Std23\tNo STUNserver for Interface " << intID << " disabling H.460.23");
+	return false;
+}
+
+bool Toolkit::H46023SameNetwork(const PIPSocket::Address & addr1, const PIPSocket::Address & addr2)
+{
+	return (m_ProxyCriterion.IsInternal(addr1) == m_ProxyCriterion.IsInternal(addr2));
+}
+#endif
+
+#ifdef HAS_H460P
+bool Toolkit::IsH460PEnabled() const 
+{
+	return m_presence.IsEnabled();
+}
+
+GkPresence & Toolkit::GetPresenceHandler() 
+{ 
+	return m_presence; 
+}
+#endif
+
 std::vector<NetworkAddress> Toolkit::GetInternalNetworks() {
 
     return GkConfig()->HasKey("ExternalIP") ? m_VirtualRouteTable.GetInternalNetworks() : m_RouteTable.GetInternalNetworks(); 
@@ -2239,8 +2476,17 @@ void Toolkit::SetGKHome(const PStringArray & home)
 		swap(m_GKHome[0], *begin);
 }
 
-int
-Toolkit::GetInternalExtensionCode( const unsigned &country,
+bool Toolkit::IsGKHome(const PIPSocket::Address & addr) const
+{
+	for (std::vector<PIPSocket::Address>::const_iterator i = m_GKHome.begin(); i != m_GKHome.end(); ++i) {
+		if (*i == addr) {
+			return true;
+		}
+	}
+	return false;
+}
+
+int Toolkit::GetInternalExtensionCode( const unsigned &country,
 				   const unsigned &extension,
 				   const unsigned &manufacturer) const
 {
@@ -2447,9 +2693,7 @@ PString Toolkit::ReadPassword(
 	return s;
 }
 
-void Toolkit::RewriteCLI(
-	SetupMsg &msg
-	) const
+void Toolkit::RewriteCLI(SetupMsg & msg) const
 {
 	m_cliRewrite->InRewrite(msg);
 }
@@ -2463,17 +2707,13 @@ void Toolkit::RewriteCLI(
 	m_cliRewrite->OutRewrite(msg, authData, addr);
 }
 
-void Toolkit::SetRerouteCauses(
-	unsigned char *causeMap
-	)
+void Toolkit::SetRerouteCauses(unsigned char *causeMap)
 {
 	memcpy(causeMap, m_causeMap, 128/8);
 }
 
 
-unsigned Toolkit::MapH225ReasonToQ931Cause(
-	int reason
-	)
+unsigned Toolkit::MapH225ReasonToQ931Cause(int reason)
 {
 	if( reason < 0 || reason > H225_ReleaseCompleteReason::e_tunnelledSignallingRejected )
 		return 0;
