@@ -10,7 +10,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef TOOLKIT_H
-#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.64 2009/06/08 10:42:11 willamowius Exp $"
+#define TOOLKIT_H "@(#) $Id: Toolkit.h,v 1.74 2010/04/29 22:22:12 willamowius Exp $"
 
 #include <vector>
 #include <map>
@@ -18,6 +18,12 @@
 #include <ptlib/sockets.h>
 #include "singleton.h"
 #include "config.h"
+#ifdef HAS_H46023
+#include "transports.h"
+#endif
+#ifdef HAS_H460P
+#include "h460presence.h"
+#endif
 
 class H225_AliasAddress;
 class H225_ArrayOf_AliasAddress;
@@ -30,7 +36,8 @@ typedef H225SignalingMsg<H225_Setup_UUIE> SetupMsg;
 struct SetupAuthData;
 
 /// Hold an address of a single host or a whole network
-struct NetworkAddress {
+class NetworkAddress {
+public:
 	/// Build an "any" address 
 	NetworkAddress();
 	/// Build a single host address
@@ -80,8 +87,9 @@ struct NetworkAddress {
 	bool operator<=(const NetworkAddress &addr) const;
 	bool operator>(const NetworkAddress &addr) const;
 	bool operator>=(const NetworkAddress &addr) const;
-	
+
 	PIPSocket::Address m_address; /// host/network address
+private:
 	PIPSocket::Address m_netmask; /// netmask for the #m_address#
 };
 
@@ -111,16 +119,17 @@ class Toolkit : public Singleton<Toolkit>
 		typedef PIPSocket::InterfaceTable InterfaceTable;
 
 	public:
-		RouteTable() : rtable_begin(0), DynExtIP(false), ExtIP("") { /* initialize later */ }
+		RouteTable() : rtable_begin(0), rtable_end(0), DynExtIP(false), ExtIP("") { /* initialize later */ }
 		virtual ~RouteTable() { ClearTable(); }
 		Address GetLocalAddress() const { return defAddr; }
-		Address GetLocalAddress(const Address &) const;
+		Address GetLocalAddress(const Address & dest) const;
 
 		void InitTable();
 		void ClearTable();
 		bool IsEmpty() const { return rtable_begin == 0; }
 
 		std::vector<NetworkAddress> GetInternalNetworks() const { return m_internalnetworks; }
+		void AddInternalNetwork(const NetworkAddress & network);
 
 	    virtual bool IsMasquerade(const PIPSocket::Address &) const { return false; }
 	protected:
@@ -147,31 +156,37 @@ class Toolkit : public Singleton<Toolkit>
 		virtual bool IsMasquerade(PIPSocket::Address &) const;
 	protected:
 		virtual bool CreateTable();
-		
 	};
 
 	RouteTable *GetRouteTable(bool = false);
 
 	class ProxyCriterion {
 		typedef PIPSocket::Address Address;
-
+		struct NetworkModes {
+			int fromExternal;	// CallRec::RoutingMode
+			int insideNetwork;	// CallRec::RoutingMode
+		};
 	public:
 		ProxyCriterion();
 		~ProxyCriterion();
 
-		bool Required(const Address &, const Address &) const;
-
+		// returns a CallRec::RoutingMode
+		int SelectRoutingMode(const Address & ip1, const Address & ip2) const;
 		void LoadConfig(PConfig *);
-
 		int IsInternal(const Address & ip) const;
 	
 	private:
+		int ToRoutingMode(const PCaselessString & mode) const;	// returns a CallRec::RoutingMode
+		NetworkAddress FindModeRule(const NetworkAddress & ip) const;
+
 		bool m_enable;
-		std::vector<NetworkAddress> m_networks;
+		std::vector<NetworkAddress> m_internalnetworks;
+		// mode selection for networks
+		std::map<NetworkAddress, NetworkModes> m_modeselection;
 	};
 
-	bool ProxyRequired(const PIPSocket::Address & ip1, const PIPSocket::Address & ip2) const
-	{ return m_ProxyCriterion.Required(ip1, ip2); }
+	int SelectRoutingMode(const PIPSocket::Address & ip1, const PIPSocket::Address & ip2) const
+	{ return m_ProxyCriterion.SelectRoutingMode(ip1, ip2); }
 
 	bool IsInternal(const PIPSocket::Address & ip1)
 	{ return (m_ProxyCriterion.IsInternal(ip1) > 0);  }
@@ -194,7 +209,7 @@ class Toolkit : public Singleton<Toolkit>
 
 	class RewriteTool {
 	public:
-		RewriteTool() : m_Rewrite(0) {}
+		RewriteTool() : m_TrailingChar(' '), m_Rewrite(0) {}
 		~RewriteTool() { delete m_Rewrite; }
 		void LoadConfig(PConfig *);
 		bool RewritePString(PString &) const;
@@ -202,7 +217,7 @@ class Toolkit : public Singleton<Toolkit>
 	private:
 		PString m_RewriteFastmatch;
 		char m_TrailingChar;
-		PString m_defaultDomain;
+		PStringArray m_defaultDomain;
 		RewriteData *m_Rewrite;
 	};
 
@@ -282,7 +297,7 @@ class Toolkit : public Singleton<Toolkit>
 		~QoSMonitor();
 	    void LoadConfig(PConfig *);
 		bool PostRecord(const std::map<PString, PString>& params);
-		bool Enabled() { return m_sqlactive; }
+		bool Enabled() const { return m_sqlactive; }
 
      private:
 	   bool m_sqlactive;
@@ -308,6 +323,7 @@ class Toolkit : public Singleton<Toolkit>
 	bool RewriteE164(H225_ArrayOf_AliasAddress & aliases);
 
 	bool RewritePString(PString & s) { return m_Rewrite.RewritePString(s); }
+	bool IsNumeric(const PString & s);
 
 	// Class to allow correct use of STL inside PDictionary type
 	class GWRewriteEntry : public PObject {
@@ -341,12 +357,23 @@ class Toolkit : public Singleton<Toolkit>
 
 	PString GetGKHome(std::vector<PIPSocket::Address> &) const;
 	void SetGKHome(const PStringArray &);
+	bool IsGKHome(const PIPSocket::Address & addr) const;
 
 	bool isBehindNAT(PIPSocket::Address &) const;
 	std::vector<NetworkAddress> GetInternalNetworks();
 #ifdef HAS_H46018
 	bool IsH46018Enabled() const { return m_H46018Enabled; }
 #endif
+#ifdef HAS_H46023
+	bool IsH46023Enabled() const;
+	void LoadH46023STUN();
+	bool GetH46023STUN(const PIPSocket::Address & addr,  H323TransportAddress & stun);
+	bool H46023SameNetwork(const PIPSocket::Address & addr1, const PIPSocket::Address & addr2);
+#endif
+#ifdef HAS_H460P
+	bool IsH460PEnabled() const;
+	GkPresence & GetPresenceHandler();
+#endif
 
 	// accessors
 	/** Accessor and 'Factory' to the static Toolkit.
@@ -594,6 +621,16 @@ private:
 #ifdef HAS_H46018
 	bool m_H46018Enabled;
 #endif
+	// is H460.23 enabled ?
+#ifdef HAS_H46023
+	bool m_H46023Enabled;
+	std::map<int,H323TransportAddress> m_H46023STUN;
+#endif
+
+#ifdef HAS_H460P
+	GkPresence m_presence;
+#endif
+
 };
 
 
