@@ -35,7 +35,7 @@
 using std::vector;
 
 /*
- * many things here should be members of Gatkeeper. 
+ * many things here should be members of Gatekeeper. 
  */
 
 PReadWriteMutex ConfigReloadMutex;
@@ -231,7 +231,24 @@ bool Gatekeeper::SetUserAndGroup(const PString& /*username*/)
 	return false;
 }
 
-#else
+// method to enable data execution prevention when supported by OS (starting with XP SP3)
+#define PROCESS_DEP_ENABLE                          0x00000001
+#define PROCESS_DEP_DISABLE_ATL_THUNK_EMULATION     0x00000002
+
+BOOL SetDEP(__in DWORD dwFlags = PROCESS_DEP_ENABLE)
+{
+	HMODULE hMod = GetModuleHandleW(L"Kernel32.dll");
+	if (!hMod)
+		return FALSE;
+	typedef BOOL (WINAPI *PSETDEP)(DWORD);
+	PSETDEP procSet = (PSETDEP)GetProcAddress(hMod,"SetProcessDEPPolicy");
+	if (!procSet)
+		return FALSE;
+	return procSet(dwFlags);
+}
+
+#else	// _WIN32
+
 #  include <pwd.h>
 
 bool Gatekeeper::SetUserAndGroup(const PString &username)
@@ -271,12 +288,6 @@ void UnixReloadHandler(int sig) // For HUP Signal
 	PTRACE(1, "GK\tGatekeeper Hangup (signal " << sig << ")");
 	ReloadHandler();
 }
-
-void DumbHandler(int sig)
-{
-	PTRACE(1, "Warning: signal " << sig << " received and ignored!");
-}
-
 #endif // _WIN32
 
 
@@ -293,6 +304,11 @@ Gatekeeper::Gatekeeper(const char * _manuf,
 	: PProcess(_manuf, _name, _majorVersion, _minorVersion, _status, _buildNumber)
 #endif
 {
+#ifdef _WIN32
+	// set data execution prevention (ignore if not available)
+	SetDEP(PROCESS_DEP_ENABLE);
+#endif
+
 #ifdef COMPILE_AS_SERVICE
 	// save original arguments
 	for (int i = 0; i < GetArguments().GetCount(); i++) {
@@ -434,7 +450,7 @@ bool Gatekeeper::InitLogging(const PArgList &args)
 #if PTRACING
 	// Syslog is the default when compiled as service, but we don't want that
 	PTrace::ClearOptions(PTrace::SystemLogStream);
-	PTrace::SetOptions(PTrace::DateAndTime | PTrace::TraceLevel);
+	PTrace::SetOptions(PTrace::DateAndTime | PTrace::TraceLevel | PTrace::FileAndLine);
 	PTrace::SetLevel(args.GetOptionCount('t'));
 	if (args.HasOption('o')) {
 		if (!SetLogFilename(args.GetOptionString('o'))) {
@@ -612,19 +628,18 @@ void Gatekeeper::Main()
 		"with the OpenH323/H323Plus and OpenSSL library.\n"
 	     << endl;
 
-#ifdef hasH460
-	// License notice
-	cout << "This program contains patent pending P2Pnat Media technology\n"
-            "under special royalty-free license to the GNU Gatekeeper Project.\n"
-		 << endl;
-#endif
-
 #ifdef HAS_H46018
 	cout << "This program contains H.460.18 and H.460.19 technology patented by Tandberg\n"
-			"licensed to the GNU Gatekeeper Project.\n"
+			"and licensed to the GNU Gatekeeper Project.\n"
 			<< endl;
 #endif
 
+#ifdef HAS_H46023
+	cout << "This program contains H.460.23 and H.460.24 technology\n"
+            "licensed to the GNU Gatekeeper Project.\n"
+		 << endl;
+#endif
+
 	// read capacity from commandline
 	int GKcapacity;
 	if (args.HasOption('b'))
@@ -802,7 +817,7 @@ void Gatekeeper::EnableLogFileRotation(
 			&Gatekeeper::RotateOnTimer, rotateTime, 60*60
 			);
 		PTRACE(5, "GK\tHourly log file rotation enabled (first "
-			"rotation sheduled at " << rotateTime
+			"rotation scheduled at " << rotateTime
 			);
 		break;
 		
@@ -815,7 +830,7 @@ void Gatekeeper::EnableLogFileRotation(
 		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
 			&Gatekeeper::RotateOnTimer, rotateTime, 60*60*24
 			);
-		PTRACE(5, "GK\tDaily rotation enabled (first rotation sheduled at " 
+		PTRACE(5, "GK\tDaily rotation enabled (first rotation scheduled at " 
 			<< rotateTime
 			);
 		break;
@@ -837,7 +852,7 @@ void Gatekeeper::EnableLogFileRotation(
 		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
 			&Gatekeeper::RotateOnTimer, rotateTime, 60*60*24*7
 			);
-		PTRACE(5, "GK\tWeekly rotation enabled (first rotation sheduled at " 
+		PTRACE(5, "GK\tWeekly rotation enabled (first rotation scheduled at " 
 			<< rotateTime
 			);
 		break;
@@ -865,7 +880,7 @@ void Gatekeeper::EnableLogFileRotation(
 		m_rotateTimer = Toolkit::Instance()->GetTimerManager()->RegisterTimer(
 			&Gatekeeper::RotateOnTimer, rotateTime
 			);
-		PTRACE(5, "GK\tMonthly rotation enabled (first rotation sheduled at " 
+		PTRACE(5, "GK\tMonthly rotation enabled (first rotation scheduled at " 
 			<< rotateTime
 			);
 		break;
