@@ -11,6 +11,15 @@
  * with the OpenH323 library.
  *
  * $Log: gksql.cxx,v $
+ * Revision 1.24  2010/04/29 22:20:42  willamowius
+ * prefer pre-increment
+ *
+ * Revision 1.23  2009/11/15 15:41:22  willamowius
+ * code cleanup
+ *
+ * Revision 1.22  2009/07/16 12:48:40  willamowius
+ * remove unused code
+ *
  * Revision 1.21  2009/05/24 20:48:26  willamowius
  * remove hacks for VC6 which isn't supported any more since quite a while
  *
@@ -142,43 +151,6 @@ bool GkSQLConnection::Initialize(
 	return Connect();	
 }
 
-bool GkSQLConnection::Initialize(
-	const char* host,
-	const char* database,
-	const char* username,
-	const char* password,
-	unsigned port,
-	int minPoolSize,
-	int maxPoolSize
-	)
-{
-	PWaitAndSignal lock(m_connectionsMutex);
-	
-	m_minPoolSize = max(minPoolSize,0);
-	if (maxPoolSize == -1)
-		m_maxPoolSize = -1;
-	else
-		m_maxPoolSize = max(maxPoolSize,m_minPoolSize);
-
-	m_host = host;
-	m_port = port;	
-
-	m_database = database;
-	m_username = username;
-	m_password = password;
-
-	if (m_host.IsEmpty() || m_database.IsEmpty()) {
-		PTRACE(1, GetName() << "\tInitialize failed: database name or host not specified!");
-		return false;
-	}
-	
-	if (!Connect()) {
-		PTRACE(1, GetName() << "\tDatabase connection failed, will rety later");
-	}
-	
-	return true;
-}
-
 bool GkSQLConnection::Connect()
 {
 	PWaitAndSignal lock(m_connectionsMutex);
@@ -213,17 +185,10 @@ void GkSQLConnection::Disconnect()
 	
 	// disconnect/delete all connections
 	PTRACE(3, GetName() << "\tDisconnecting all SQL connections in pool");
-	for(iterator Iter = m_idleConnections.begin();
-		Iter != m_idleConnections.end(); Iter++) {
+	for(iterator Iter = m_idleConnections.begin(); Iter != m_idleConnections.end(); ++Iter) {
 		delete *Iter;
 	}
 	m_idleConnections.clear();
-//	for(iterator Iter = m_busyConnections.begin();
-//		Iter != m_busyConnections.end(); Iter++) {
-//		delete *Iter;
-//	}
-//	m_busyConnections.clear();
-	// TODO:: delete waiting ?
 	m_connected = false;
 }
 
