@@ -11,6 +11,12 @@
  * with the OpenH323 library.
  *
  * $Log: gksql_odbc.cxx,v $
+ * Revision 1.13.6.1  2010/05/25 15:47:07  willamowius
+ * fix 64bit compile with gcc 4.4.3
+ *
+ * Revision 1.14  2010/05/25 15:46:29  willamowius
+ * fix 64bit compile with gcc 4.4.3
+ *
  * Revision 1.13  2009/02/09 13:25:59  willamowius
  * typo in comment
  *
@@ -492,7 +498,7 @@ GkSQLResult* GkODBCConnection::ExecuteQuery(
 		return new GkODBCResult(r, errmsg + ", query: " + queryStr);
 	}
 	
-	SQLINTEGER rows = 0;
+	SQLLEN rows = 0;
 	
 	if (columns == 0) {
 		if (nodata) {
@@ -537,7 +543,7 @@ GkSQLResult* GkODBCConnection::ExecuteQuery(
 		GkSQLResult::ResultRow * row = new GkSQLResult::ResultRow(columns);
 		
 		for (SQLUSMALLINT i = 1; i <= columns; ++i) {
-			SQLINTEGER indicator;
+			SQLLEN indicator;
 			char data[512];
 			/* retrieve column data as a string */
 			SQLRETURN result = SQLGetData(stmt, i, SQL_C_CHAR, data, sizeof(data), &indicator);
