@@ -17,7 +17,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef JOB_H
-#define JOB_H "@(#) $Id: job.h,v 1.12 2009/05/28 11:00:07 willamowius Exp $"
+#define JOB_H "@(#) $Id: job.h,v 1.13 2009/09/17 15:43:21 willamowius Exp $"
 
 #include "name.h"
 
@@ -189,7 +189,7 @@ protected:
 		const PTimeInterval& timeout
 		) 
 	{ 
-		return m_sync.Wait(timeout) ? true : false; 
+		return m_sync.Wait(timeout); 
 	}
 
 	/// Send a signal to the waiting task
