@@ -9,7 +9,7 @@
 //
 // This work is published under the GNU Public License (GPL)
 // see file COPYING for details.
-// We also explicitely grant the right to link this code
+// We also explicitly grant the right to link this code
 // with the OpenH323 library.
 //
 // initial author: Chin-Wei Huang <cwhuang@linux.org.tw>
@@ -18,7 +18,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef NAME_H
-#define NAME_H "@(#) $Id: name.h,v 1.6 2006/04/14 13:56:19 willamowius Exp $"
+#define NAME_H "@(#) $Id: name.h,v 1.8 2009/02/09 15:26:21 willamowius Exp $"
 
 #if PTRACING
 
@@ -35,6 +35,8 @@ public:
 	NamedObject(
 		const NamedObject& obj
 		) : m_name((const char*)(obj.m_name)) {}
+		
+	virtual ~NamedObject() {}
 
 	/// assignment operator for proper PString assignment
 	NamedObject& operator=(
