@@ -12,6 +12,12 @@
  * with the OpenH323 library.
  *
  * $Log: radauth.cxx,v $
+ * Revision 1.51  2010/02/10 02:09:52  willamowius
+ * remove unused variables
+ *
+ * Revision 1.50  2009/10/25 22:13:09  willamowius
+ * allow to set proxy mode through 'proxy' variable in Radius h323-ivr-in attribute (thanks to Yuriy Georgiewskiy)
+ *
  * Revision 1.49  2009/05/24 20:48:26  willamowius
  * remove hacks for VC6 which isn't supported any more since quite a while
  *
@@ -903,7 +909,36 @@ int RadAuthBase::Check(
 			}
 		}
 	}
-	
+
+	// process h323-ivr-in=proxy attribute
+	if (result) {
+		attr = response->FindVsaAttr(RadiusAttr::CiscoVendorId, 
+			RadiusAttr::CiscoVSA_AV_Pair
+			);
+		while (attr != NULL) {
+			PINDEX index;
+			value = attr->AsCiscoString();
+			if (value.Find("h323-ivr-in=") == 0 
+				&& ((index = value.Find("proxy:")) != P_MAX_INDEX)) {
+				index += strlen("proxy:");
+				const PINDEX semicolonpos = value.Find(';', index);
+				value = value.Mid(index, semicolonpos == P_MAX_INDEX
+					? P_MAX_INDEX : (semicolonpos-index)
+					);
+				if (!value) {
+					authData.m_proxyMode = Toolkit::AsBool(value)
+						? CallRec::ProxyEnabled : CallRec::ProxyDisabled;
+					PTRACE(5, "RADAUTH\t" << GetName() << " - proxy mode "
+						<< (authData.m_proxyMode == CallRec::ProxyEnabled ? "enabled" : "disabled")
+					);
+				}
+			}
+			attr = response->FindVsaAttr(RadiusAttr::CiscoVendorId, 
+				RadiusAttr::CiscoVSA_AV_Pair, attr
+				);
+		}
+	}
+
 	if (result)
 		result = OnReceivedPDU(*response, arqPdu, authData);
 	else
@@ -924,7 +959,7 @@ int RadAuthBase::Check(
 	H225_Setup_UUIE& setupBody = setup.GetUUIEBody();
 	const bool hasCall = authData.m_call.operator->() != NULL;
 	PIPSocket::Address addr;
-	endptr callingEP, calledEP;
+	endptr callingEP;
 	
 	if (hasCall)
 		callingEP = authData.m_call->GetCallingParty();
@@ -1192,6 +1227,35 @@ int RadAuthBase::Check(
 		}
 	}
 
+	// process h323-ivr-in=proxy attribute
+	if (result) {
+		attr = response->FindVsaAttr(RadiusAttr::CiscoVendorId, 
+			RadiusAttr::CiscoVSA_AV_Pair
+			);
+		while (attr != NULL) {
+			PINDEX index;
+			value = attr->AsCiscoString();
+			if (value.Find("h323-ivr-in=") == 0 
+				&& ((index = value.Find("proxy:")) != P_MAX_INDEX)) {
+				index += strlen("proxy:");
+				const PINDEX semicolonpos = value.Find(';', index);
+				value = value.Mid(index, semicolonpos == P_MAX_INDEX
+					? P_MAX_INDEX : (semicolonpos-index)
+					);
+				if (!value) {
+					authData.m_proxyMode = Toolkit::AsBool(value)
+						? CallRec::ProxyEnabled : CallRec::ProxyDisabled;
+					PTRACE(5, "RADAUTH\t" << GetName() << " - proxy mode "
+						<< (authData.m_proxyMode == CallRec::ProxyEnabled ? "enabled" : "disabled")
+						);
+				}
+			}
+			attr = response->FindVsaAttr(RadiusAttr::CiscoVendorId, 
+				RadiusAttr::CiscoVSA_AV_Pair, attr
+				);
+		}
+	}
+
 	if (result)
 		result = OnReceivedPDU(*response, setup, authData);
 	else
