@@ -11,6 +11,15 @@
  * with the OpenH323 library.
  *
  * $Log: radproto.cxx,v $
+ * Revision 1.48  2010/04/30 11:45:57  willamowius
+ * fix VS2010 warnings
+ *
+ * Revision 1.47  2009/11/15 15:45:52  willamowius
+ * code cleanup
+ *
+ * Revision 1.46  2009/09/17 15:43:21  willamowius
+ * code cleanup
+ *
  * Revision 1.45  2009/05/28 10:08:16  willamowius
  * use the more portable ios::fmtflags instead of _Ios_Fmtflags
  *
@@ -570,7 +579,7 @@ void RadiusAttr::PrintOn(
 	ostream &strm   /// Stream to print the object into.
     ) const
 {
-	const int indent = strm.precision() + 2;
+	const std::streamsize indent = strm.precision() + 2;
 
 	if (!IsValid()) {
 		strm << "(Invalid) {\n";
@@ -859,7 +868,7 @@ void RadiusPDU::PrintOn(
 	ostream& strm /// Stream to print the object into.
 	) const
 {
-	const int indent = strm.precision() + 2;
+	const std::streamsize indent = strm.precision() + 2;
 
 	strm << ((!IsValid()) ? "(Invalid) {\n" : "{\n");
 	
@@ -889,7 +898,7 @@ void RadiusPDU::PrintOn(
 	else {
 		strm << setw(indent+13) << "attributes = " << numAttributes << " elements {\n";
 
-		const int aindent = indent + 2;
+		const std::streamsize aindent = indent + 2;
 
 		const RadiusAttr* attr = GetAttr();
 		PINDEX i = 0;
@@ -1553,7 +1562,6 @@ RadiusSocket::RadiusSocket(
 		Close();
 	}
 	
-	int i;
 	PRandom random;
 	const unsigned _id_ = random;
 	m_oldestId = m_nextId = (BYTE)(_id_^(_id_>>8)^(_id_>>16)^(_id_>>24));
@@ -1565,6 +1573,7 @@ RadiusSocket::RadiusSocket(
 		memset(m_syncPointMap, 0, sizeof(m_syncPointMap));
 		memset(m_idTimestamps, 0, sizeof(m_idTimestamps));
 
+		int i;
 		for (i = 0; i < 256; i++)
 			m_readSyncPointIndices[i] = P_MAX_INDEX;
 
@@ -1945,7 +1954,7 @@ RadiusClient::RadiusClient(
 #if PTRACING
 	if (PTrace::CanTrace(4)) {
 		ostream& s = PTrace::Begin(4, __FILE__, __LINE__);
-		const int indent = s.precision() + 2;
+		const std::streamsize indent = s.precision() + 2;
 		s << "RADIUS\tCreated instance of RADIUS client (local if: "
 			<< m_localAddress << ", default ports: " << m_authPort << ',' 
 			<< m_acctPort << ") for RADIUS servers group:";
@@ -1979,7 +1988,7 @@ RadiusClient::RadiusClient(
 	m_numRetries(config.GetInteger(sectionName, "RequestRetransmissions",
 		DefaultRetries)),
 	m_roundRobinServers(config.GetBoolean(
-		sectionName, "RoundRobinServers", TRUE) ? true : false),
+		sectionName, "RoundRobinServers", TRUE)),
 	m_localAddress(INADDR_ANY)
 {
 	GetServersFromString(config.GetString(sectionName, "Servers", ""));
@@ -2023,7 +2032,7 @@ RadiusClient::RadiusClient(
 #if PTRACING
 	if (PTrace::CanTrace(4)) {
 		ostream& os = PTrace::Begin(4, __FILE__, __LINE__);
-		const int indent = os.precision() + 2;
+		const std::streamsize indent = os.precision() + 2;
 		os << "RADIUS\tCreated instance of RADIUS client (local if: "
 			<< m_localAddress << ", default ports: " << m_authPort << ',' 
 			<< m_acctPort << ") for RADIUS servers group:";
