@@ -10,7 +10,7 @@
  * We also explicitly grant the right to link this code
  * with the OpenH323 library.
  *
- * @(#) $Id: sigmsg.cxx,v 1.7 2009/04/20 09:28:49 willamowius Exp $
+ * @(#) $Id: sigmsg.cxx,v 1.8 2009/09/17 15:43:21 willamowius Exp $
  */
 #include <ptlib.h>
 #include <ptlib/sockets.h>
@@ -102,12 +102,12 @@ bool SignalingMsg::Encode(PBYTEArray &buffer)
 		strm.CompleteEncoding();
 		m_q931->SetIE(Q931::UserUserIE, strm);
 	}
-	return m_q931->Encode(buffer) ? true : false;
+	return m_q931->Encode(buffer);
 }
 
 bool SignalingMsg::Decode(const PBYTEArray & buffer)
 {
-	return m_q931->Decode(buffer) ? true : false;
+	return m_q931->Decode(buffer);
 }
 
 SignalingMsg* SignalingMsg::Create(
