@@ -7,11 +7,11 @@
  *
  * This work is published under the GNU Public License (GPL)
  * see file COPYING for details.
- * We also explicitely grant the right to link this code
+ * We also explicitly grant the right to link this code
  * with the OpenH323 library.
  */
 #ifndef SIGMSG_H
-#define SIGMSG_H "@(#) $Id: sigmsg.h,v 1.7 2008/02/13 13:32:25 willamowius Exp $"
+#define SIGMSG_H "@(#) $Id: sigmsg.h,v 1.9 2009/04/20 09:28:49 willamowius Exp $"
 
 
 #include <ptlib/sockets.h>
@@ -90,6 +90,9 @@ public:
 	bool Encode(
 		PBYTEArray &buffer /// buffer to hold the encoded message
 		);
+	bool Decode(
+		const PBYTEArray & buffer /// buffer holding the encoded message
+		);
 			
 	/// factory constructor for signaling messages
 	static SignalingMsg* Create(
