@@ -11,6 +11,15 @@
  * with the OpenH323 library.
  *
  * $Log: sqlacct.cxx,v $
+ * Revision 1.21  2010/03/24 10:09:49  willamowius
+ * don't warn about empty query for events whose result is ignored anyway
+ *
+ * Revision 1.20  2010/02/10 02:09:52  willamowius
+ * remove unused variables
+ *
+ * Revision 1.19  2009/12/02 14:32:13  willamowius
+ * add new accounting events: AcctAlert, AcctRegister, AcctUnregister
+ *
  * Revision 1.18  2009/05/24 20:48:26  willamowius
  * remove hacks for VC6 which isn't supported any more since quite a while
  *
@@ -177,6 +186,20 @@ SQLAcct::SQLAcct(
 			<< m_stopQueryAlt);
 	}
 
+	m_alertQuery = cfg->GetString(cfgSec, "AlertQuery", "");
+	if (!m_alertQuery) {
+		PTRACE(4, "GKACCT\t" << GetName() << " alert query: " << m_alertQuery);
+	}
+
+	m_registerQuery = cfg->GetString(cfgSec, "RegisterQuery", "");
+	if (!m_registerQuery) {
+		PTRACE(4, "GKACCT\t" << GetName() << " registration query: " << m_registerQuery);
+	}
+	m_unregisterQuery = cfg->GetString(cfgSec, "UnregisterQuery", "");
+	if (!m_unregisterQuery) {
+		PTRACE(4, "GKACCT\t" << GetName() << " un-registration query: " << m_unregisterQuery);
+	}
+
 	vector<PIPSocket::Address> interfaces;
 	Toolkit::Instance()->GetGKHome(interfaces);
 	if (interfaces.empty()) {
@@ -235,13 +258,18 @@ GkAcctLogger::Status SQLAcct::Log(
 	else if (evt == AcctStop) {
 		query = m_stopQuery;
 		queryAlt = m_stopQueryAlt;
+	} if (evt == AcctAlert)
+		query = m_alertQuery;
+
+	if (query.IsEmpty() && (evt == AcctAlert)) {
 	}
-	
+
 	if (query.IsEmpty()) {
-		PTRACE(2, "GKACCT\t" << GetName() << " failed to store accounting "
-			"data (event: " << evt << ", call: " << callNumber 
-			<< "): SQL query is empty"
-			);
+		if (evt != AcctAlert) {
+			PTRACE(2, "GKACCT\t" << GetName() << " failed to store accounting "
+				"data (event: " << evt << ", call: " << callNumber 
+				<< "): SQL query is empty");
+		}
 		return Fail;
 	}
 
@@ -303,6 +331,74 @@ GkAcctLogger::Status SQLAcct::Log(
 	return succeeded ? Ok : Fail;
 }
 
+GkAcctLogger::Status SQLAcct::Log(
+	GkAcctLogger::AcctEvent evt, 
+	const endptr& ep
+	)
+{
+	if ((evt & GetEnabledEvents() & GetSupportedEvents()) == 0)
+		return Next;
+		
+	if (!ep) {
+		PTRACE(1, "GKACCT\t" << GetName() << " - missing call info for event " << evt);
+		return Fail;
+	}
+	
+	const PString epid = ep->GetEndpointIdentifier().GetValue();
+
+	if (m_sqlConn == NULL) {
+		PTRACE(2, "GKACCT\t" << GetName() << " failed to store accounting "
+			"data (event: " << evt << ", endpoint: " << epid
+			<< "): SQL connection not active"
+			);
+		return Fail;
+	}
+	
+	PString query;
+	if (evt == AcctRegister)
+		query = m_registerQuery;
+	else if (evt == AcctUnregister)
+		query = m_unregisterQuery;
+
+	if (query.IsEmpty()) {
+		return Fail;
+	}
+
+	std::map<PString, PString> params;
+	SetupAcctEndpointParams(params, ep);
+	GkSQLResult* result = m_sqlConn->ExecuteQuery(query, params);
+	if (result == NULL) {
+		PTRACE(2, "GKACCT\t" << GetName() << " failed to store accounting "
+			"data (event: " << evt << ", endpoint: " << epid
+			<< "): timeout or fatal error");
+	}
+	
+	if (result) {
+		if (result->IsValid()) {
+			if (result->GetNumRows() < 1) {
+				PTRACE(4, "GKACCT\t" << GetName() << " failed to store accounting "
+					"data (event: " << evt << ", endpoint: " << epid
+					<< "): no rows have been updated"
+					);
+				delete result;
+				result = NULL;
+			}
+		} else {
+			PTRACE(2, "GKACCT\t" << GetName() << " failed to store accounting "
+				"data (event: " << evt << ", endpoint: " << epid
+				<< "): (" << result->GetErrorCode() << ") "
+				<< result->GetErrorMessage()
+				);
+			delete result;
+			result = NULL;
+		}
+	}
+
+	const bool succeeded = result != NULL && result->IsValid();	
+	delete result;
+	return succeeded ? Ok : Fail;
+}
+
 PString SQLAcct::GetInfo()
 {
 	PString result;
