@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef YASOCKET_H
-#define YASOCKET_H "@(#) $Id: yasocket.h,v 1.34 2009/05/28 11:00:07 willamowius Exp $"
+#define YASOCKET_H "@(#) $Id: yasocket.h,v 1.37 2009/09/17 15:43:21 willamowius Exp $"
 
 #include <list>
 #include <vector>
@@ -49,8 +49,8 @@ public:
 
 	void SetPort(WORD pt) { port = pt; }
 	WORD GetPort() const { return port; }
-	virtual void GetLocalAddress(Address &) const;
-	virtual void GetLocalAddress(Address &, WORD &) const;
+	virtual PBoolean GetLocalAddress(Address &) const;
+	virtual PBoolean GetLocalAddress(Address &, WORD &) const;
 	bool SetOption(int, int, int = SOL_SOCKET);
 	bool SetOption(int, const void *, int, int = SOL_SOCKET);
 	bool GetOption(int, int &, int = SOL_SOCKET);
@@ -119,7 +119,7 @@ private:
 	sockaddr_in peeraddr;
 };
 
-class YaUDPSocket : public YaSocket {
+class YaUDPSocket : public YaSocket, public PObject {
 public:
 	YaUDPSocket();
 
@@ -299,8 +299,8 @@ public:
 	virtual bool TransmitData(const PBYTEArray &);
 	virtual bool TransmitData(const BYTE *, int);
 
-	bool IsSocketOpen() const { return self->IsOpen() ? true : false; }
-	bool CloseSocket() { return IsSocketOpen() ? (self->Close() ? true : false) : false; }
+	bool IsSocketOpen() const { return self->IsOpen(); }
+	bool CloseSocket() { return IsSocketOpen() ? self->Close() : false; }
 
 	virtual bool Flush();
 	bool CanFlush() const { return (qsize > 0) && IsSocketOpen(); }
